#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import distutils.spawn
import sys


from launchpadlib import credentials
from lazr.restfulclient import errors

HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.join(HERE, '../ci-utils'))
from ci_utils import dump_stack

web_root = 'https://launchpad.net/'

creds_path = os.path.abspath('lp_creds.txt')

creds_template = '''
export CI_OAUTH_CONSUMER_KEY={}
export CI_OAUTH_TOKEN={}
export CI_OAUTH_TOKEN_SECRET={}
# Can't find lp user in the creds, it's bzr lp-login anyway
export CI_LAUNCHPAD_USER=`bzr lp-login`
# This should be a team that owns the ppa build pool
export CI_LAUNCHPAD_PPA_OWNER=`bzr lp-login`
'''


def create():
    creds = credentials.Credentials('ci-airline')
    url = creds.get_request_token(web_root=web_root)

    if distutils.spawn.find_executable('xdg-open'):
        handler = 'xdg-open'
    elif distutils.spawn.find_executable('open'):
        handler = 'open'
    else:
        sys.stderr.write('Could not find a program to handle URLs.')
        sys.exit(1)
    subprocess.call([handler, url])
    sys.stderr.write("Authenticate from your browser (just opened),"
                     " choose 'Change anything'\nthen press enter\n")
    sys.stdin.readline()
    try:
        creds.exchange_request_token_for_access_token(web_root=web_root)
    except errors.HTTPError as e:
        if e.response['status'] == '403':
            sys.stderr.write('Ok, not creating credentials then\n')
            sys.exit(1)
        raise

    args = (creds.consumer.key,
            creds.access_token.key,
            creds.access_token.secret)

    with open(creds_path, 'wb') as f:
        f.write(creds_template.format(*args))
    sys.stderr.write('Credentials created in {}\n'.format(creds_path))

    return 0


def main(args):
    create()


if __name__ == '__main__':
    dump_stack.install_stack_dump_signal()
    sys.exit(main(sys.argv[1:]))
