#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import os
import sys

from datetime import datetime


HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(HERE, '..', 'ci-utils'))
from ci_utils import get_nagios_dir


def parse_arguments(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-t', required=True, type=int, dest='timeout',
        help=('Interval to consider when checking health check output '
              'age, in seconds.'))
    parser.add_argument('component_health_filename')
    return parser.parse_args(args)


def main(args=None):
    args = parse_arguments(args)

    timeout = args.timeout
    result_file = os.path.join(
        get_nagios_dir(), args.component_health_filename)
    now = datetime.now()

    # Check if file exists
    if not os.path.exists(result_file):
        print("CRITICAL: health check result file not found.")
        sys.exit(2)

    # Check if file is newer than -t seconds
    last_modified = datetime.fromtimestamp(os.path.getmtime(result_file))
    if (now - last_modified).seconds > timeout:
        print("WARNING: health check last run finished more than {} seconds "
              "ago.".format(timeout))
        sys.exit(1)

    # Health check always expects 'OK' or 'FAILED' in the first line of the
    # result file.
    result_file_contents = open(result_file).readlines()
    if not 'OK' in result_file_contents[0]:
        # Printing only the first line of the result file because nagios
        # displays limited output on its interface (one line). Concatenating
        # the whole file on a single message would produce a very confusing
        # output. According to #is, current practice is display only first
        # line, and ssh into the machine in case more details are needed.
        result = ""
        if len(result_file_contents) > 1:
            result = ": {}".format(result_file_contents[1])
        print("CRITICAL: health check didn't return OK{}".format(result))
        sys.exit(2)

    print("OK: tests passed.")
    sys.exit(0)


if __name__ == "__main__":
    main()
