# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock
import os
import shutil
import tempfile
import unittest

from ci_utils import gpg
from ci_utils.testing import (
    get_test_file_path,
    TESTING_KEYRING_DATA,
)


class BsbTestBase(unittest.TestCase):

    def setUp(self):
        super(BsbTestBase, self).setUp()

        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)

        _p = mock.patch('ci_utils.unit_config.get_auth_config')
        self.addCleanup(_p.stop)
        _p.start().return_value = {}

        _p = mock.patch('ci_utils.unit_config.get')
        self.addCleanup(_p.stop)
        _p.start()

    def copy_test_files(self, files):
        """Copy given upload-files to the testing directory."""
        for fname in files:
            src = get_test_file_path(fname)
            dst = os.path.join(self.tmpdir, fname)
            shutil.copyfile(src, dst)

    def rename_test_files(self, src, dst):
        """Rename given upload-files in the testing directory."""
        shutil.move(
            os.path.join(self.tmpdir, src), os.path.join(self.tmpdir, dst)
        )

    def assertSigned(self, fname):
        """Asserts the given upload file is signed by the test key."""
        with gpg.GnupgEnvironment() as gnupg_env:
            gnupg_env.importKeyring(TESTING_KEYRING_DATA)
            [test_key] = gnupg_env.listKeys()
            path = os.path.join(self.tmpdir, fname)
            with open(path) as fd:
                sig = gnupg_env.gnupg.verify_file(fd)
        self.assertEqual(sig.fingerprint, test_key['fingerprint'])
