# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Configuration options related to the britney proxy."""

import os


from uciconfig import (
    options,
    stacks,
    stores,
)


BritneyStore = stores.FileStore


def user_config_dir():
    conf_dir = os.environ.get('XDG_CONFIG_HOME',
                              os.path.expanduser('~/.config'))
    return conf_dir


class BritneyStack(stacks.Stack):
    """Per-user options."""

    def __init__(self):
        """Make a new stack for britney interactions with the uci engine.

        The options are searched in ~/.config/uci-britney.conf.
        """
        upath = os.path.join(user_config_dir(), 'uci-britney.conf')
        user_store = self.get_shared_store(BritneyStore(upath))
        super(BritneyStack, self).__init__(
            # We only care about the no name section
            [stacks.NameMatcher(user_store, None).get_sections],
            user_store, mutable_section_id=None)


def register(option):
    options.option_registry.register(option)


register(options.Option('uci.rabbit.host',
                        default=options.MANDATORY,
                        help_string='''\
The host:port address for the uci-britney rabbit server.
'''))
register(options.Option('uci.rabbit.user',
                        default=options.MANDATORY,
                        help_string='''\
The user name for the uci-britney rabbit server.
'''))
register(options.Option('uci.rabbit.password',
                        default=options.MANDATORY,
                        help_string='''\
The user password for the uci-britney rabbit server.
'''))


def amqp_config(conf):
    """Create the rabbit config from the britney one."""
    class AmqpConfig(object):

        def __init__(self):
            self.AMQP_HOST = conf.get('uci.rabbit.host')
            self.AMQP_VHOST = '/'
            self.AMQP_USER = conf.get('uci.rabbit.user')
            self.AMQP_PASSWORD = conf.get('uci.rabbit.password')

    return AmqpConfig
