# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest


from ucitests import fixtures


from britney import config


class TestConfigDir(unittest.TestCase):

    def setUp(self):
        super(TestConfigDir, self).setUp()
        fixtures.set_uniq_cwd(self)
        fixtures.isolate_from_env(self, dict(HOME=self.uniq_dir,
                                             XDG_CONFIG_HOME=None))

    def test_default(self):
        conf_dir = os.path.join(self.uniq_dir, '.config')
        self.assertEqual(conf_dir, config.user_config_dir())

    def test_xdg_config_home_override(self):
        xdg_dir = os.path.join(self.uniq_dir, 'xdg')
        os.environ['XDG_CONFIG_HOME'] = xdg_dir
        self.assertEqual(xdg_dir, config.user_config_dir())


class TestConfig(unittest.TestCase):

    def setUp(self):
        super(TestConfig, self).setUp()
        fixtures.set_uniq_cwd(self)
        fixtures.isolate_from_env(self, dict(HOME=self.uniq_dir,
                                             XDG_CONFIG_HOME=None))
        self.config = config.BritneyStack()

    def test_empty_config_errors(self):
        with self.assertRaises(Exception) as cm:
            self.config.get('uci.rabbit.host')
        self.assertEqual('uci.rabbit.host must be set.', unicode(cm.exception))
