# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from cStringIO import StringIO
import unittest


from ucitests import assertions


from britney import post_request
from ci_utils.testing import fixtures


class TestCliRun(unittest.TestCase):
    "Smoke blackbox tests."""

    def setUp(self):
        super(TestCliRun, self).setUp()
        self.out = StringIO()
        self.err = StringIO()

    def test_run_no_args_errors(self):
        with self.assertRaises(SystemExit):
            post_request.cli_run([], self.out, self.err)


class TestOptionParsing(unittest.TestCase):

    def setUp(self):
        super(TestOptionParsing, self).setUp()
        self.out = StringIO()
        self.err = StringIO()

    def parse_args(self, args):
        ns = post_request.PostRequestArgParser().parse_args(
            args, self.out, self.err)
        return ns

    def test_default_values(self):
        ns = self.parse_args(['trusty', 'amd64', 'libpng'])
        self.assertEqual(None, ns.output)
        self.assertEqual(None, ns.report)
        self.assertEqual('', self.out.getvalue())
        self.assertEqual('', self.err.getvalue())


class TestPost(unittest.TestCase):

    def test_post_simple(self):
        queue = fixtures.FakeTaskQueue('that_queue')
        post_request.post(queue, 'trusty', 'amd64', 'libpng')
        assertions.assertLength(self, 1, queue.msgs)
        self.assertEqual(dict(series='trusty', architecture='amd64',
                              package='libpng'),
                         queue.msgs[0])

    def test_post_with_report(self):
        queue = fixtures.FakeTaskQueue('that_queue')
        post_request.post(queue, 'trusty', 'amd64', 'libpng',
                          report='this_queue')
        assertions.assertLength(self, 1, queue.msgs)
        self.assertEqual(dict(series='trusty', architecture='amd64',
                              package='libpng', report_queue='this_queue'),
                         queue.msgs[0])
