# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest


from ucitests import fixtures as uci_fixtures


from britney import process_results
from ci_utils.testing import (
    features,
    fixtures,
)


class TestBritneyTestRunnerArtifactMapper(unittest.TestCase):

    def setUp(self):
        super(TestBritneyTestRunnerArtifactMapper, self).setUp()
        # Set some defaults to make the tests less verbose
        self.release = 'utopic'
        self.architecture = 'amd64'
        self.time_stamp = '20141001_000000'
        self.host_name = 'localhost'

    def get_mapper(self, package):
        mapper = process_results.BritneyTestRunnerArtifactMapper(
            self.release, self.architecture, package, self.time_stamp,
            self.host_name)
        return mapper

    def test_package_name(self):
        mapper = self.get_mapper('hello')
        self.assertEquals(
            'utopic/amd64/h/hello/20141001_000000_localhost/hello.log',
            mapper.get_artifact_name('hello.log'))

    def test_lib_package_name(self):
        mapper = self.get_mapper('libpng')
        self.assertEquals(
            'utopic/amd64/libp/libpng/20141001_000000_localhost/libpng.log',
            mapper.get_artifact_name('libpng.log'))


# FIXME: This works around https://app.asana.com/0/8499154990155/14496314215756
# 'unit_config as a fixture for tests that requires it' -- vila 2014-09-25
@features.requires(features.nova_creds)
class TestBritneyResultHandler(unittest.TestCase):

    def setUp(self):
        super(TestBritneyResultHandler, self).setUp()
        uci_fixtures.set_uniq_cwd(self)
        self.tr_ds_path = os.path.join(self.uniq_dir, 'tr_data_store')
        self.tr_data_store = fixtures.FakeDataStore(self.tr_ds_path)

        def test_runner_data_store_factory(ticket_id):
            return self.tr_data_store

        self.britney_ds_path = os.path.join(self.uniq_dir,
                                            'britney_data_store')
        self.britney_data_store = fixtures.FakeDataStore(self.britney_ds_path)
        self.handler = process_results.BritneyResultHandler(
            self.britney_data_store, test_runner_data_store_factory,
            _time_stamp='20141001_000000')

    def test_status_message(self):
        params = dict(state='STATUS')
        # Since the handler does nothing, this is mostly a smoke test
        self.handler.handle(params)

    def test_completed_message(self):
        request = dict(ticket_id='not-used', series='utopic',
                       architecture='amd64',
                       package_list=['fake'])
        fake_content = 'fake.log content\n'
        fake_url = self.tr_data_store.put_file('fake.log', fake_content,
                                               'text/plain')
        fake_br_content = 'fake.britney.results content\n'
        fake_br_url = self.tr_data_store.put_file('fake.britney.results',
                                                  fake_br_content,
                                                  'text/plain')

        params = dict(state='COMPLETED', request=request,
                      artifacts=[dict(name='fake.log', reference=fake_url,
                                      type='LOGS'),
                                 dict(name='fake.britney.results',
                                      reference=fake_br_url,
                                      type='RESULTS')])
        self.handler.handle(params)
        log_name = self.handler.mapper.get_artifact_name('fake.log')
        actual_content = self.britney_data_store.get_file(log_name)
        self.assertEquals(fake_content, actual_content)
        br_name = self.handler.mapper.get_artifact_name('fake.britney.results')
        actual_br_content = self.britney_data_store.get_file(br_name)
        self.assertEquals(fake_br_content, actual_br_content)
        # The test runner data store has been removed
        self.assertFalse(os.path.exists(self.tr_ds_path),
                         '{} should have been deleted'.format(self.tr_ds_path))
