#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import sys
import base64


import charmhelpers.core.hookenv
import charmhelpers.core.host


hooks = charmhelpers.core.hookenv.Hooks()


def juju_info(msg, level='INFO'):
    charmhelpers.core.hookenv.log(msg, level)


def _service_dir(unit):
    unit = unit.split('/')[0]
    for x in (':', '"', "'"):
        unit = unit.replace(x, '_')
    config = charmhelpers.core.hookenv.config()
    install_root = config['install_root']
    return os.path.join(install_root, unit)


def _install_key(key_value, keys_dir, key_file):
    key_path = os.path.join(keys_dir, key_file)
    if not key_value:
        juju_info('{} skipped (empty content)'.format(key_path))
        return
    with open(key_path, 'w') as f:
        f.write(base64.b64decode(key_value))
    juju_info('{} installed'.format(key_path))


def _install_keys(unit):
    config = charmhelpers.core.hookenv.config()
    service_dir = _service_dir(unit)
    keys_dir = os.path.join(service_dir, 'keys')
    if os.path.exists(keys_dir):
        shutil.rmtree(keys_dir)
        juju_info('Keys removed from {}'.format(keys_dir))
    try:
        charmhelpers.core.host.mkdir(keys_dir, perms=0755)
        _install_key(config['gpg-public-keyring'], keys_dir, 'gpg.pub')
        _install_key(config['gpg-secret-keyring'], keys_dir, 'gpg.sec')
        _install_key(config['ssh-public-key'], keys_dir, 'id_rsa.pub')
        _install_key(config['ssh-private-key'], keys_dir, 'id_rsa')
    except IOError as exc:
        juju_info('Cannot install keys: {}'.format(exc))
        if os.path.exists(keys_dir):
            shutil.rmtree(keys_dir)


@hooks.hook('config-changed')
def config_changed():
    juju_info('Configuration changed.')
    relations = charmhelpers.core.hookenv.relations_of_type('juju-info')
    if not relations:
        juju_info("No container yet, nothing to do")
        return
    unit = relations[0].get('__unit__')
    _install_keys(unit)


@hooks.hook('juju-info-relation-joined')
def joined():
    unit = charmhelpers.core.hookenv.remote_unit()
    juju_info('Container joined: {}'.format(unit))
    _install_keys(unit)


if __name__ == "__main__":
    hooks.execute(sys.argv)
