#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import os
import sys

from charmhelpers.core import hookenv

hooks = hookenv.Hooks()
log = hookenv.log


def get_service_dir():
    unit_name = hookenv.local_unit()
    sanitized_unit_name = unit_name.split('/')[0]
    return os.path.join('/srv', sanitized_unit_name)


@hooks.hook('intercom-relation-changed')
@hooks.hook('intercom-relation-joined')
def update_intercom(remove=False):
    """Update the inter-components map.

    It maintains a map of <unit> -> (<hostname>, <port>) components
    that may exchange data directly.

    It consumes 'http' (hostname:port) interface.

    The inter-components map is saved as 'intercom.json' in the
    application DIR in JSON format.
    """
    intercom_path = os.path.join(get_service_dir(), 'intercom.json')

    if os.path.exists(intercom_path):
        registry = json.load(open(intercom_path))
    else:
        registry = {}

    relation_id = hookenv.relation_id()
    unit = hookenv.remote_unit()
    hostname = hookenv.relation_get('hostname')
    port = hookenv.relation_get('port')

    if remove:
        log('Intercom unit \'{}\' ({}) departed.'.format(unit, hostname))
        if registry.get(relation_id) is not None:
            del registry[relation_id]
    else:
        log('Intercom unit \'{}\' ({}) joined.'.format(unit, hostname))
        registry[relation_id] = {
            'unit': unit,
            'hostname': hostname,
            'port': port,
        }

    with open(intercom_path, 'w') as fd:
        json.dump(registry, fd)


@hooks.hook()
def intercom_relation_broken():
    update_intercom(remove=True)


if __name__ == "__main__":
    hooks.execute(sys.argv)
