# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import shutil
import tempfile


# Avoid the "No handlers could be found for logger XXX" one-off warning. We
# don't use logging (for now) in *this* module, but submodules needs handler to
# be defined for their parent (i.e. 'ci_utils.amqp_utils' requires a defined
# logger for 'ci_utils')
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())


# Current supported API Versions
API_VERSIONS = {
    'image_builder': '20140721',
}


SUPPORTED_SERIES = [
    'precise',
    'trusty',
    'utopic',
    'vivid',
]


SUPPORTED_POCKETS = {
    '': 'Release',
    # '-updates': 'Updates',
}


def get_ci_utils_dir():
    """Utility function for facilitating tests.

    Return '/srv/<service>/code/current/ci-utils/ci_utils' in production.
    """
    return os.path.abspath(os.path.dirname(__file__))


def get_service_dir():
    """Returns the deployment service dir (e.g. '/srv/ci-airline-foo/')."""
    path = os.path.join(
        get_ci_utils_dir(), os.pardir, os.pardir, os.pardir, os.pardir)
    return os.path.abspath(path)


def get_logs_dir():
    """Returns the deployment logs_dir path (e.g. '/srv/<service>/logs')."""
    return os.path.join(get_service_dir(), 'logs')


def get_nagios_dir():
    """Returns the deployment nagios_dir path.

    E.g. '/srv/<service>/nagios').
    """
    return os.path.join(get_service_dir(), 'nagios')


def valid_suites():
    suites = []
    for pocket in SUPPORTED_POCKETS.iterkeys():
        for series in SUPPORTED_SERIES:
            suites.append('{}{}'.format(series, pocket))
    return suites


class TmpDir(object):
    '''Create a temporary directory with a context manager'''

    def __init__(self, suffix='', prefix='tmp', dir=None):
        self.path = tempfile.mkdtemp(suffix, prefix, dir)

    def __enter__(self):
        return self.path

    def __exit__(self, exc, value, tb):
        shutil.rmtree(self.path)
