# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Access to juju environment definitions."""

import os


import yaml


def home():
    return os.environ.get('JUJU_HOME',
                          os.path.join(os.path.expanduser('~'), '.juju'))


def environments_from_yaml():
    envs_path = os.path.join(home(), 'environments.yaml')
    if not os.path.exists(envs_path):
        return None

    with open(envs_path) as f:
        full_envs = yaml.safe_load(f.read())
    return full_envs


def environment_name():
    # From the env variable
    name = os.environ.get('JUJU_ENV')
    if name:
        return name

    # From $JUJU_HOME/current-environment (set by 'juju switch').
    cur_env_path = os.path.join(home(), 'current-environment')
    if os.path.exists(cur_env_path):
        with open(cur_env_path) as f:
            name = f.read().strip('\n')
            return name

    # From the default key in the environments.yaml file
    full_envs = environments_from_yaml()
    if full_envs is None:
        return None

    return full_envs.get('default', None)


def environment(env_name):
    """Get the juju specified env as a dict.

    :param env_name: The name of the juju environment in the
        environments.yaml file.
    """
    full_env = environments_from_yaml()
    if full_env is None:
        return None
    env_defs = full_env.get('environments', None)
    if env_defs is None:
        return None
    return env_defs.get(env_name, None)
