# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import textwrap

from ci_utils import unit_config

from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import UnencryptedFileCredentialStore


def _get_credential_store():
    creds = '/tmp/ci-engine.lpcreds'
    if not os.path.exists(creds):
        with open(creds, 'w') as f:
            os.fchmod(f.fileno(), 0700)
            template = textwrap.dedent('''\
            [1]
            consumer_key = {}
            consumer_secret =
            access_token = {}
            access_secret = {}
            ''')
            f.write(template.format(unit_config.get('oauth_consumer_key'),
                                    unit_config.get('oauth_token'),
                                    unit_config.get('oauth_token_secret')))

    return UnencryptedFileCredentialStore(creds)


def lp_login():
    lib = '/tmp/launchpadlib'
    store = _get_credential_store()
    return Launchpad.login_with('ci-engine', 'production', version='devel',
                                credential_store=store, launchpadlib_dir=lib)


def get_lp_user_name():
    return unit_config.get('launchpad_user')
