#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import os.path
import sys

import data_store


def parse_arguments():
    parser = argparse.ArgumentParser(
        description='Utility for uploading files into a datastore.',
        epilog='''The public/private state applies to the container, not the
        individual files and can be changed through additional uploads.''')
    parser.add_argument('-c', '--component',
                        help='The component name for the cantainer.',
                        required=True)
    parser.add_argument('-i', '--identifier',
                        help='The datastore identifier.',
                        default=None)
    parser.add_argument('-p', '--public',
                        help='''Set the container to public (default is
                        private).''',
                        action='store_true',
                        default=False)
    parser.add_argument('file', nargs='*')
    return parser.parse_args()


def main():
    args = parse_arguments()
    try:
        auth_config = {
            'auth_url': os.environ['OS_AUTH_URL'],
            'auth_user': os.environ['OS_USERNAME'],
            'auth_password': os.environ['OS_PASSWORD'],
            'auth_tenant_name': os.environ['OS_TENANT_NAME']}
    except KeyError as e:
        print('Missing key [{}].'.format(e))
        print('Did you remember to source your nova environment file?')
        return 1

    datastore = data_store.DataStore(args.component, auth_config,
                                     args.identifier, public=args.public)
    for upload_file in args.file:
        base_name = os.path.basename(upload_file)
        with open(upload_file) as in_file:
            data = in_file.read()
        location = datastore.put_file(base_name, data)
        print(location)
    return 0


if __name__ == '__main__':
    sys.exit(main())
