# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
"""Ubuntu CI Engine common testing infrastructure."""

import mock
import os
import unittest

from ci_utils.gpg import GnupgEnvironment

HERE = os.path.dirname(os.path.abspath(__file__))
# Absolute path to access a collection of testing files (mainly source
# package upload files).
TEST_DATA_ROOT = os.path.join(HERE, 'data')


# ASCII 'armored' keyring with the public GPG keys used in tests.
TESTING_KEYRING_PATH = os.path.join(TEST_DATA_ROOT, 'testing_keyring.pub')
TESTING_KEYRING_DATA = open(TESTING_KEYRING_PATH).read()

# ASCII 'armored' private GPG keys used in tests.
TESTING_PRIVATE_KEYRING_PATH = os.path.join(
    TEST_DATA_ROOT, 'testing_keyring.sec')
TESTING_PRIVATE_KEYRING_DATA = open(TESTING_PRIVATE_KEYRING_PATH).read()


def get_test_file_path(filename):
    """Return the absolute path of a given testing file."""
    return os.path.join(TEST_DATA_ROOT, filename)


class TestCaseWithGnupg(unittest.TestCase):
    """Provides the necessary setup for testing with custom keyring."""

    def setUp(self):
        super(TestCaseWithGnupg, self).setUp()
        # Isolates an empty (mocked _unit_config) GNUPGHOME and rely
        # on unittest cleanup to restore its previous state.
        self.gnupg = GnupgEnvironment(prefix='testing-gnupg-')
        with mock.patch(
                'ci_utils.unit_config.get_auth_config') as mock_config:
            mock_config.return_value = {}
            self.gnupg.isolate(context_manager=True)
        self.addCleanup(self.gnupg.restore)
        # Overcautious kering import, we currently have only one
        # public key in our testing keyring.
        self.assertEquals(len(self.gnupg.listKeys()), 0)
        self.gnupg.importKeyring(TESTING_KEYRING_DATA)
        self.assertEquals(len(self.gnupg.listKeys()), 1)
