# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.

# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
"""Ubuntu CI Engine deployer helpers."""
import os
import yaml


def load_configs(deployer_dir, deployer_filenames):
    """Load and merge multiple deployment configurations.

    Agregates 'services' and 'relations' of the same deployments.
    """
    full_config = {}
    for config_name in deployer_filenames:
        config_path = os.path.join(deployer_dir, config_name)
        with open(config_path) as fd:
            config = yaml.safe_load(fd.read())
        for deployment_name, context in config.iteritems():
            if deployment_name not in full_config.keys():
                full_config[deployment_name] = context
                continue
            full_context = full_config[deployment_name]
            full_context['services'].update(context['services'])
            full_context['relations'].extend(context['relations'])

    return full_config
