# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest


from ci_utils import data_store


class TestDataStoreConfig(unittest.TestCase):

    def test_valid_auth_config(self):
        data_store.DataStore.validate_auth_config({
            'auth_url': 'http://example.com',
            'auth_user': 'user',
            'auth_password': 'pass',
            'auth_tenant_name': 'tenant',
            'auth_region': 'region',
        })

    def test_invalid_auth_config(self):
        with self.assertRaises(data_store.DataStoreException):
            data_store.DataStore.validate_auth_config({})


class TestDataStoreFileName(unittest.TestCase):

    def setUp(self):
        super(TestDataStoreFileName, self).setUp()
        # We won't use any method that requires the auth config so we don't
        # provide any.
        self.ds = data_store.DataStore('foo', None)

    def test_abspath(self):
        self.assertEquals('foo', self.ds.file_name('/baz/bar/foo'))

    def test_relpath(self):
        self.assertEquals('foo', self.ds.file_name('../baz/bar/foo'))

    def test_basename(self):
        self.assertEquals('foo', self.ds.file_name('foo'))


if __name__ == "__main__":
    unittest.main()
