#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import unittest
import yaml

from ucitests import fixtures as uci_fixtures
from ci_utils.testing.deployer import load_configs


TEST_CONFIGS = {
    'test-a.yaml': {
        'test-deployment': {
            'services': {
                'service-a': {
                    'charm': 'a_charm',
                }
            },
            'series': 'A_SERIES',
            'relations': [
                ['a', 'b'],
            ],
        },
    },
    'test-b.yaml': {
        'test-deployment': {
            'services': {
                'service-b': {
                    'charm': 'b_charm',
                },
            },
            'series': 'A_SERIES',
            'relations': [
                ['c', 'd'],
            ],
        },
    },
}


class TestDeployer(unittest.TestCase):
    """Test deployer helpers."""

    def setUp(self):
        """Create a disposable testing-jail."""
        super(TestDeployer, self).setUp()
        uci_fixtures.set_uniq_cwd(self)

    def create_config(self, name, content):
        """Helper for creating YAML configurations."""
        config_path = os.path.join(self.uniq_dir, name)
        with open(config_path, 'w') as fd:
            yaml.safe_dump(content, fd)

    def test_load_empty_single_config(self):
        # An empty configuration is loaded as such.
        config_name = 'test-empty.yaml'
        config_content = {}
        self.create_config(config_name, config_content)
        self.assertEquals(
            {}, load_configs(self.uniq_dir, [config_name]))

    def test_load_single_config(self):
        # Single configurations are loaded correctly.
        name, content = list(TEST_CONFIGS.iteritems())[0]
        self.create_config(name, content)
        self.assertEquals(
            content, load_configs(self.uniq_dir, [name]))

    def test_load_multiple_config(self):
        # Multiple deployment configurations are merged.
        for name, content in TEST_CONFIGS.iteritems():
            self.create_config(name, content)
        self.assertEquals(
            {
                'test-deployment': {
                    'series': 'A_SERIES',
                    'services': {
                        'service-a': {'charm': 'a_charm'},
                        'service-b': {'charm': 'b_charm'},
                    },
                    'relations': [['c', 'd'], ['a', 'b']],
                },
            },
            load_configs(self.uniq_dir, TEST_CONFIGS.keys()))


if __name__ == "__main__":
    unittest.main()
