#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import image_store
from keystoneclient.exceptions import AuthorizationFailure
import unittest
import mock


class TestImageStore(unittest.TestCase):
    # Do not attempt to talk to a keystone server.
    @mock.patch('keystoneclient.httpclient.HTTPClient.authenticate',
                side_effect=AuthorizationFailure())
    def test_bad_auth(self, *args):
        auth_data = {
            'auth_url': 'https://keystone.canonistack.canonical.com:443/v2.0/',
            'auth_user': 'bad data',
            'auth_password': 'bad data',
            'auth_tenant_name': 'bad data'
        }
        with self.assertRaises(image_store.ImageStoreException):
            image_store.ImageStore(auth_data)

    def test_missing_auth(self):
        with self.assertRaises(image_store.ImageStoreException):
            image_store.ImageStore({})

    def test_bad_server(self):
        auth_data = {
            'auth_url': 'badurl',
            'auth_user': 'bad data',
            'auth_password': 'bad data',
            'auth_tenant_name': 'bad data'
        }
        with self.assertRaises(image_store.ImageStoreException):
            image_store.ImageStore(auth_data)


class TestUciImageName(unittest.TestCase):

    def test_valid_britney_image(self):
        self.assertEqual(
            'uci/britney/precise-amd64.img',
            image_store.uci_image_name('britney', 'precise', 'amd64'))

    def test_valid_cloud_image(self):
        self.assertEqual(
            'uci/cloudimg/precise-amd64.img',
            image_store.uci_image_name('cloudimg', 'precise', 'amd64'))

    def test_invalid_image(self):
        with self.assertRaises(ValueError) as cm:
            image_store.uci_image_name('I-dont-exist', 'precise', 'amd64')
        self.assertEqual('Invalid image domain', unicode(cm.exception))


if __name__ == "__main__":
    unittest.main()
