
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Tests for deployment paths helpers."""
import mock
import os
import shutil
import tempfile
import unittest

from ci_utils import (
    get_logs_dir,
    get_nagios_dir,
    get_service_dir,
)


class TestPaths(unittest.TestCase):

    def setUp(self):
        """Setup a fake deployment 'root' path in a temporary directory.

        Mocks 'ci_utils.get_service_dir' to return it.
        """
        super(TestPaths, self).setUp()
        self.root = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.root)
        ci_utils_path = os.path.join(
            self.root, 'srv/ci-service/code/current/ci-utils/ci_utils')
        os.makedirs(ci_utils_path)
        _m = mock.patch('ci_utils.get_ci_utils_dir')
        mocked_ci_utils_dir = _m.start()
        mocked_ci_utils_dir.return_value = ci_utils_path
        self.addCleanup(_m.stop)

    def assertPath(self, expected, path):
        """Local assertion for eliminating 'root' path from checks."""
        clean_path = path.replace(self.root, '')
        self.assertEqual(expected, clean_path)

    def test_ci_utils_dir(self):
        # Simply proves the testing mocking strategy corresponds to reality.
        from ci_utils import get_ci_utils_dir
        self.assertPath(
            '/srv/ci-service/code/current/ci-utils/ci_utils',
            get_ci_utils_dir())

    def test_service_dir(self):
        self.assertPath('/srv/ci-service', get_service_dir())

    def test_logs_dir(self):
        self.assertPath('/srv/ci-service/logs', get_logs_dir())

    def test_nagios_dir(self):
        self.assertPath('/srv/ci-service/nagios', get_nagios_dir())


if __name__ == "__main__":
    unittest.main()
