
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import mock
import time
import unittest
from ci_utils import stats


def mock_func(self, *args, **kwargs):
    pass


class TestStats(unittest.TestCase):

    def setUp(self):
        self.metric_1_name = "test_metric_1"
        self.metric = stats.Metric(self.metric_1_name)

    @mock.patch('ci_utils.stats.Metric._publish')
    def test_start(self, mock_func):
        self.assertEqual(self.metric_1_name, self.metric.name)

    def test_bad_name(self):
        mo = mock.MagicMock()
        mo.txstatsd.client.UdpStatsDClient.connect = mock_func
        mo.txstatsd.metrics.extendedmetrics.ExtendedMetrics.timing = mock_func
        m = stats.Metric(None)
        self.assertRaises(
            stats.BadName,
            m.stop
        )

    @mock.patch('ci_utils.stats.Metric._publish')
    def test_lap(self, mock_func):
        self.metric.lap("mid-point")

        current_duration = self.metric.lap_duration

        self.metric.lap("mid-point-2")

        self.assertNotEqual(current_duration, self.metric.lap_duration)

    @mock.patch('ci_utils.stats.Metric._publish')
    def test_lap_creates_duration(self, mock_func):

        self.metric.lap("mid-point")
        self.assert_(self.metric.duration)

    @mock.patch('ci_utils.stats.Metric._publish')
    def test_lap_uses_lap_start_time(self, mock_func):

        time.sleep(0.1)
        self.metric.lap("mid-point")
        self.assertNotEqual(self.metric.start_time, self.metric.lap_start_time)

    @mock.patch('ci_utils.stats.Metric._publish')
    def test_stop(self, mock_func):

        self.metric.stop()
        self.assert_(self.metric.duration)


if __name__ == "__main__":
    unittest.main()
