# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lazr.enum import DBEnumeratedType, DBItem


class Item(DBItem):

    def __int__(self):
        return int(self.value)


class SubTicketWorkflowStep(DBEnumeratedType):

    NEW = Item(000, "New")
    QUEUED = Item(100, "Queued")
    PKG_BUILDING = Item(200, "Package building")
    COMPLETED = Item(1000, "Completed")

    def __str__(self):
        return str(self.value)


class SubTicketWorkflowStepStatus(DBEnumeratedType):

    NEW = Item(000, "New")
    PKG_BUILDING_WAITING = Item(210, "Not started")
    PKG_BUILDING_INPROGRESS = Item(220, "In progress")
    PKG_BUILDING_COMPLETED = Item(230, "Completed")
    PKG_BUILDING_FAILED = Item(240, "Failed")

    def __str__(self):
        return str(self.value)


class TicketWorkflowStep(DBEnumeratedType):

    NEW = Item(000, "New")

    SILO_CREATING = Item(10, "Silo creating")
    SILO_BUILDING = Item(20, "Silo building")
    SILO_TESTING = Item(30, "Silo testing")
    SILO_PUBLISHING = Item(40, "Silo publishing")

    QUEUED = Item(100, "Queued")
    PPA_CREATING = Item(150, "PPA creating")
    PKG_BUILDING = Item(200, "Package building")
    PKG_VALIDATION = Item(250, "Package validation")
    IMAGE_BUILDING = Item(300, "Image building")
    IMAGE_TESTING = Item(400, "Image testing")
    PKG_PUBLISHING = Item(500, "Package publishing")

    FAILED = Item(999, "Failed")
    COMPLETED = Item(1000, "Completed")

    def __str__(self):
        return str(self.value)


class TicketWorkflowStepStatus(DBEnumeratedType):
    QUEUED = Item(0, 'Not started')
    PAUSED = Item(10, 'Paused')
    CONTINUE = Item(20, 'Continue')
    INPROGRESS = Item(100, 'In progress')
    COMPLETED = Item(200, 'Completed')
    FAILED = Item(300, 'Failed')

    def __str__(self):
        return str(self.value)


class ReviewStatus(DBEnumeratedType):

    PENDING = Item(0, "Pending")
    APPROVED = Item(10, "Approved")
    DISAPPROVED = Item(100, "Disapproved")

    def __str__(self):
        return str(self.value)


# we have workflow steps known to the lander (lander_service_wrapper). We
# also have steps associated in the ticket-system. This provides a mapping
# to translate between them
STEP_TO_WORKFLOW_MAPPING = {
    'ppacreator': TicketWorkflowStep.PPA_CREATING.value,
    'bsbuilder': TicketWorkflowStep.PKG_BUILDING.value,
    'validator': TicketWorkflowStep.PKG_VALIDATION.value,
    'image_builder': TicketWorkflowStep.IMAGE_BUILDING.value,
    'test_runner': TicketWorkflowStep.IMAGE_TESTING.value,
    'publisher': TicketWorkflowStep.PKG_PUBLISHING.value,
}
