# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

import yaml


HERE = os.path.abspath(os.path.dirname(__file__))


_unit_config = None


def _load():
    path = os.path.join(HERE, '../../../../unit_config')
    # TODO remove once all charms have converted to new code layout
    # Fallback to old location configs (inside code tree) for local branches
    # testing and charms transition.
    if not os.path.exists(path):
        path = os.path.abspath(os.path.join(HERE, '../../unit_config'))
    with open(path) as f:
        global _unit_config
        _unit_config = yaml.safe_load(f)


def get(key):
    if not _unit_config:
        _load()
    return _unit_config[key]


def get_auth_config():
    if not _unit_config:
        _load()
    return _unit_config


def is_cloud(identifier, string=None):
    """Are we running against/on the specified cloud.

    :param identifier: The magic string to use as a check against the auth URL.
    :param string: The authentication URL to check (defaults to OS_AUTH_URL).
    """
    if string is None:
        string = os.environ.get('OS_AUTH_URL', None)
    if string and identifier in string:
        return True
    return False


def is_hpcloud(string=None):
    """Are we running against/on HP cloud.

    :param string: The authentication URL to check (defaults to OS_AUTH_URL).
    """
    return is_cloud('hpcloud', string)


def is_canonistack(string=None):
    """Are we running against/on canonistack cloud.

    :param string: The authentication URL to check (defaults to OS_AUTH_URL).
    """
    return is_cloud('canonistack', string)
