#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging


from ci_utils import (
    amqp_utils,
    amqp_worker,
)
from coverageretriever.retrieval import Retrieval, RetrievalFailed


class CoverageRetrieverWorker(amqp_worker.AMQPWorker):

    def __init__(self):
        super(CoverageRetrieverWorker, self).__init__('coverageretriever')

    def handle_request(self, params, logger):
        """Snip coverage.xml from incoming build log and put in swift.

        :param params: dictionary of params from incoming message
        :param logger: logging.Logger for reporting
        """
        ticket_id = params['ticket_id']
        data_store = self._create_data_store(ticket_id)
        # stash our common objects for convenience
        retrieval = Retrieval(ticket_id, logger, data_store)
        try:
            build_log_path = retrieval.try_extract_build_log_path(params)
            log_text = retrieval.try_get_build_log_from_path(build_log_path)
            coverage_xml = retrieval.try_extract_coverage_xml(log_text)
            coverage_xml_link = retrieval.try_put_coverage_xml(coverage_xml)
        except RetrievalFailed as e:
            return e.return_tuple
        return retrieval.report_succeeded(ticket_id, coverage_xml_link)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format="%(asctime)s %(levelname)s %(message)s")
    CoverageRetrieverWorker().main(amqp_utils.COVERAGE_RETRIEVER_QUEUE)
