# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from coverageretriever.tests import CoverageRetrieverTestCase
from coverageretriever.retrieval import (
    generate_and_put_artifact,
)


class GenerateAndPutArtifactTestCase(CoverageRetrieverTestCase):

    def test_logging_defaults(self):
        """Generate and put (and return) a log artifact."""
        with self.patch_create_data_store():
            artifact = generate_and_put_artifact(
                self.fake_data_store,
                message="Have a nice day!",
            )
        self.assertEqual("Have a nice day!", artifact['message'])
        self.assertEqual('LOGS', artifact['type'])
        self.assertEqual('coverage-retriever.output.log', artifact['name'])

    def test_coverage_xml_result(self):
        """Generate and put a coverage.xml artifact."""
        with self.patch_create_data_store():
            artifact = generate_and_put_artifact(
                self.fake_data_store,
                name='coverage.xml',
                reference='fake-swift-link',
                type='RESULTS',
            )
        self.assertEqual('coverage.xml', artifact['name'])
        self.assertEqual('RESULTS', artifact['type'])
        self.assertEqual('fake-swift-link', artifact['reference'])

    def test_message_is_not_none(self):
        """If no message, then substitute our header message."""
        with self.patch_create_data_store():
            artifact = generate_and_put_artifact(
                self.fake_data_store,
                message='not-none',
            )
        self.assertEqual('not-none', artifact['message'])
