# -*- coding: utf-8 -*-
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Swift datastore base classes."""
from __future__ import unicode_literals

from swiftclient import client


class BasicDataStore(object):
    """Swift basic data store utility."""

    def __init__(self, options):
        """Immediately retrieve and store Swift authenticated URL."""
        self.options = options
        self._setup_auth()

    def _setup_auth(self):
        if self.options.api_version == '1':
            self.url, self.token = client.get_auth_1_0(
                self.options.auth_url,
                self.options.username,
                self.options.password,
                False,
            )
            return
        self.url, self.token = client.get_auth(
            self.options.auth_url,
            self.options.username,
            self.options.password,
            tenant_name=self.options.tenant,
            os_options={'region_name': self.options.region},
            auth_version=self.options.api_version,
        )
