# -*- coding: utf-8 -*-
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Swift datastore base tests."""
from __future__ import unicode_literals

from mock import patch
from testtools import TestCase

from gatekeeper.datastore.tests import FakeOptions
from gatekeeper.datastore.base import BasicDataStore


class TestBasicDataStore(TestCase):
    """Tests basic DataStore."""

    def test_instantiation_v1(self):
        # DataStore authenticates, using the client.get_auth_1_0(), when
        # instantiated.
        options = FakeOptions(version='1')
        with patch('swiftclient.client.get_auth_1_0') as mock_auth:
            mock_auth.return_value = ('A_URL', 'A_TOKEN')
            datastore = BasicDataStore(options)
        mock_auth.assert_called()
        self.assertEquals('A_URL', datastore.url)
        self.assertEquals('A_TOKEN', datastore.token)

    def test_instantiation_v2(self):
        # DataStore authenticates, using the client.get_auth(), when
        # instantiated.
        options = FakeOptions()
        with patch('swiftclient.client.get_auth') as mock_auth:
            mock_auth.return_value = ('A_URL', 'A_TOKEN')
            datastore = BasicDataStore(options)
        mock_auth.assert_called()
        self.assertEquals('A_URL', datastore.url)
        self.assertEquals('A_TOKEN', datastore.token)
