# -*- coding: utf-8 -*-
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Gatekeeper WSGI app entry point.

Exposes Swift "Sandbox" (supervised container + tempurls) funtionality
via a REST API.
Access Swift using the environment configuration options (OS_*)
"""
from __future__ import unicode_literals

from gatekeeper.datastore import DataStore
from gatekeeper.datastore.options import DataStoreOptions
from gatekeeper.resources.root import make_app


def environment_datastore_factory(klass):
    """Rely on ``DataStoreOptions` to use 'OS_*' variables."""
    # XXX cprov 2014-04-25: it will not work for HP until we
    # can (artificially) define variable for tenant_id, access_key_id
    # and secret_key values needed for tempurl.
    options = DataStoreOptions()
    return DataStore(options)


def unit_config_datastore_factory(klass):
    """Uses the CI-Airline 'unit_config' information."""
    from ci_utils.unit_config import get_auth_config
    config = get_auth_config()
    options = DataStoreOptions(
        auth_url=config.get('auth_url'),
        username=config.get('auth_user'),
        password=config.get('auth_password'),
        tenant=config.get('auth_tenant_name'),
        region=config.get('auth_region'),
        access_key_id=config.get('auth_access_key_id'),
        tenant_id=config.get('auth_tenant_id'),
        secret_key=config.get('tempurl_signing_key'),
        expires=config.get('tempurl_expires'),
        public_host=config.get('tempurl_public_host'),
    )
    return DataStore(options)

app = make_app(unit_config_datastore_factory)
# app = make_app(environment_datastore_factory)
