#!/bin/sh
# create an uci/britney/${release}-${arch}.img nova image based on a generic
# image and adt-setup-vm
# Author: Martin Pitt <martin.pitt@ubuntu.com>
set -eu
BASEIMG="${1:-}"

if [ -z "$BASEIMG" ]; then
    echo "Usage: $0 <base image name> [key name]" >&2
    exit 1
fi

opts=""
[ -z "${2:-}" ] || opts="$opts --key_name $2"
[ -z "${EPHEMERAL_CLOUD_NET_ID:-}" ] || opts="$opts --nic net-id=$EPHEMERAL_CLOUD_NET_ID"

# unbreak my server option :-(
userdata=`mktemp`
trap "rm $userdata" EXIT TERM INT QUIT PIPE
/bin/echo -e "#cloud-config\nmanage_etc_hosts: True" > $userdata

# create new instance
INSTNAME="$BASEIMG-adt-prepare"
nova boot --flavor m1.smaller --image $BASEIMG --user-data $userdata $opts --poll $INSTNAME

# determine IP address
ipaddr=$(nova show --minimal $INSTNAME | awk 'BEGIN {FS="|"} /network/ {n=split($3,i,/,\s*/); gsub(" ", "", i[n]); print i[n]}')
SSH_CMD="ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no ubuntu@$ipaddr"
echo "Waiting for ssh (may cause some error messages)..."
timeout 5m sh -c "while ! $SSH_CMD true; do sleep 5; done"

echo "Waiting until cloud-init is done..."
timeout 10m $SSH_CMD 'while [ ! -e /var/lib/cloud/instance/boot-finished ]; do sleep 1; done'

# Hack to create a vivid image from an utopic one

# $SSH_CMD 'sudo sed -i "s/utopic/vivid/g" /etc/apt/sources.list /etc/apt/sources.list.d/*.list || true'
# $SSH_CMD sudo apt-get update
# $SSH_CMD sudo apt-get -y dist-upgrade

echo "Running adt setup script..."
$SSH_CMD sudo sh -x < /usr/share/autopkgtest/adt-setup-vm
release=$($SSH_CMD lsb_release -sc)
arch=$($SSH_CMD dpkg --print-architecture)

echo "Powering off to get a clean file system..."
$SSH_CMD sudo poweroff
while ! nova show --minimal $INSTNAME | grep -q 'vm_state.*stopped'; do sleep 1; done

# determine new image name and build it
IMAGE_NAME="uci/britney/${release}-${arch}.img"
nova image-create --show --poll $INSTNAME "$IMAGE_NAME"
nova delete $INSTNAME
echo
echo "Successfully built image $IMAGE_NAME"
