#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging


from ci_utils import amqp_utils, amqp_worker

from imagebuilder.cloud_image import modify_image
from imagebuilder.cloud_image import get_image


class ImageBuildWorker(amqp_worker.AMQPWorker):

    def __init__(self):
        super(ImageBuildWorker, self).__init__('image_build')

    def engine_health(self):
        status = super(ImageBuildWorker, self).engine_health()
        status.add_image_store_health()
        return status

    def handle_request(self, params, log):

        image = params['image']
        trigger = params['progress_trigger']

        def status_cb(msg):
            log.info(msg)
            amqp_utils.progress_update(trigger, {'message': msg})

        #Check API version
        api = params.get('api_version')
        if api == '20140721':
            image_id = get_image(image, status_cb)
            retval = {'image_id': image_id}
        else:
            repos = params['ppa_list']
            series = params['series']
            packages = params['package_list']
            ticket_id = params['ticket_id']
            ds = self._create_data_store(ticket_id)

            image_id, location = modify_image(image, repos, series, packages,
                                              ds, status_cb)
            retval = {'image_id': image_id}
            self._create_artifact(image_id, location, retval, type='IMAGE')

        return amqp_utils.progress_completed, retval


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format="%(asctime)s %(levelname)s %(message)s")
    ImageBuildWorker().main(amqp_utils.IMAGE_BUILDER_QUEUE)
