# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock
import unittest

from ci_utils.json_status import JSONStatus
from imagebuilder import run_worker


class TestImagebuildWorker(unittest.TestCase):
    def setUp(self):
        super(TestImagebuildWorker, self).setUp()
        self.worker = run_worker.ImageBuildWorker()

    @mock.patch('ci_utils.json_status.JSONStatus.add_deployed_revision')
    @mock.patch('ci_utils.json_status.JSONStatus.add_image_store_health')
    @mock.patch('ci_utils.json_status.JSONStatus.add_data_store_health')
    def test_engine_health(self, mocked_ds, mocked_is, mocked_rev):
        # `ImageBuildWorker` implements Engine-Health interface with
        # deployed-revision, data-store and image-store connectivity.
        self.assertIsInstance(self.worker.engine_health(), JSONStatus)
        mocked_ds.assert_called_once()
        mocked_is.assert_called_once()
        mocked_rev.assert_called_once()

    @mock.patch('imagebuilder.run_worker.modify_image')
    @mock.patch('ci_utils.amqp_utils.progress_completed')
    @mock.patch('ci_utils.amqp_worker.AMQPWorker._create_data_store')
    def test_data_store_container(self, mocked_create_data_store,
                                  mocked_progress_completed,
                                  mocked_modify_image):
        params = {
            'image': 'http://fake/location',
            'ticket_id': 1,
            'ppa_list': [],
            'package_list': [],
            'series': 'saucy',
            'progress_trigger': 'ticket_1_trigger'
        }

        mocked_log = mock.MagicMock()
        mocked_ds = mock.MagicMock()
        mocked_modify_image.return_value = ('fake_image_id', 'fake_location')
        mocked_create_data_store.return_value = mocked_ds

        self.worker.handle_request(params, mocked_log)

        mocked_create_data_store.assert_called_once_with(params['ticket_id'])

    @mock.patch('imagebuilder.run_worker.get_image')
    @mock.patch('ci_utils.amqp_utils.progress_completed')
    def test_worker_get_image(self, mocked_progress_completed,
                              mocked_get_image):
        params = {
            'api_version': '20140721',
            'image': 'http://fake/location',
            'ticket_id': 1,
            'series': 'saucy',
            'progress_trigger': 'ticket_1_trigger'
        }

        mocked_log = mock.MagicMock()
        mocked_get_image.return_value = ('fake_image_id')

        self.worker.handle_request(params, mocked_log)

        mocked_get_image.assert_called_once()
