#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import cgi
import os
import json
import subprocess
import sys

# enable us to get quality backtraces
import cgitb
cgitb.enable()


HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.append(os.path.join(HERE, '../../ci-utils'))
from ci_utils import json_status


try:
    # Infer the service name from the directory this lives under
    # Service names sometimes change, and this is hard to find and know to
    # fix otherwise
    service = os.path.abspath(os.path.join(HERE, '../../../../'))
    service = os.path.basename(service)
    state = subprocess.check_output(
        ['/sbin/status', service], stderr=subprocess.STDOUT)
except subprocess.CalledProcessError as e:
    state = e.output
state = state.strip()

print('Content-type: application/json')
print('')
status = json_status.JSONStatus()
status.add_true_false('lander worker', state, 'start/running' in state)
status.add_deployed_revision()

relations_dir = '../../../../juju-relations'
exist = os.path.exists(relations_dir)
if exist:
    ppa = ts = False
    for f in os.listdir(relations_dir):
        content = open(os.path.join(relations_dir, f)).read()
        if 'ts-django' in content:
            ts = True
    if not ts:
        status.add_true_false('ticket-system relation', 'missing', False)
else:
    status.add_true_false('missing juju-relations', exist, exist)

params = cgi.parse()
if params.get('format') == ['jsonp']:
    cb = params.get('callback', ['json_callback'])[0]
    print('%s(%s)' % (cb, json.dumps(status.results)))
else:
    print(json.dumps(status.results))
