#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import json


def parse_arguments():
    parser = argparse.ArgumentParser(
        description='Merges a service result into the prior results.')
    parser.add_argument('--service',
                        required=True,
                        help='The service name which generated the results.')
    parser.add_argument('--result-file',
                        required=True,
                        help='The results from the given service.')
    parser.add_argument('--prior-file',
                        default=None,
                        help='Results file from prior calls.')
    parser.add_argument('--output-file',
                        required=True,
                        help='The new merged results file.')
    return parser.parse_args()


def main():
    args = parse_arguments()

    if args.prior_file:
        with open(args.prior_file) as prior_file:
            data = json.loads(prior_file.read())
    else:
        data = {}
    with open(args.result_file) as result_file:
        data[args.service] = json.loads(result_file.read())

    with open(args.output_file, 'w') as output_file:
        output_file.write(json.dumps(data))
        output_file.write('\n')


if __name__ == '__main__':
    main()
