# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import mock
import unittest


from ucitests import fixtures
from lander import run_worker


class TestRunWorker(unittest.TestCase):
    '''Sanity tests of the run_worker.py script.'''

    def setUp(self):
        super(TestRunWorker, self).setUp()
        fixtures.set_uniq_cwd(self)
        fixtures.patch(self, run_worker, 'SERVICE_PATH', self.uniq_dir)
        with open('relation1', 'w') as f:
            f.write('bsbuilder: 1.2.3.4')
        with open('relation2', 'w') as f:
            f.write('image_builder: 1.2.3.41')

    def test_no_relations(self):
        fixtures.patch(self, run_worker, 'SERVICE_PATH', '/does not exsit')
        self.assertIsNone(run_worker.get_service_url('foo', 80))

    def test_get_service_url(self):
        self.assertIsNone(run_worker.get_service_url('foo', 80))
        self.assertEqual(
            'http://1.2.3.4:80', run_worker.get_service_url('bsbuilder', 80))

    @mock.patch('lander.run_worker._process_ticket')
    def test_on_message_good(self, process):
        msg = mock.Mock()
        msg.body = json.dumps({})
        lander = run_worker.Lander('ts_url', None)
        lander.on_message(msg)
        self.assertEqual(1, process.call_count)
        self.assertEqual(1, msg.channel.basic_ack.call_count)

    @mock.patch('lander.run_worker._process_ticket')
    def test_on_bad_message(self, process):
        '''ensure we ack the message so it won't be retried'''
        lander = run_worker.Lander('ts_url', None)
        msg = mock.Mock()
        msg.body = 'invalid json'
        lander.on_message(msg)
        self.assertEqual(0, process.call_count)
        self.assertEqual(1, msg.channel.basic_ack.call_count)

    @mock.patch('lander.run_worker._store_artifacts')
    @mock.patch('lander.run_worker._update_local_result')
    @mock.patch('subprocess.call')
    def test_process_resume(self, call, update, store):
        data = {'id': 1, 'resume': True}
        call.return_value = 12
        run_worker._process_ticket('tsurl', data, './')
        self.assertIn('--resume', call.call_args[0][0])
        self.assertEqual(1, update.call_count)
