# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tempfile
import json
import os
import shutil
import unittest

import mock

import lander
from lander import (
    add_status, log_status, LOG_STATUS_FILENAME, STATUS_FILENAME
)


def get_logs_dir():
    return tempfile.mkdtemp()


class TestStatus(unittest.TestCase):
    '''Sanity tests of the status logging feature.'''

    @mock.patch('ci_utils.get_logs_dir')
    def setUp(self, mock_get_logs_dir):
        self.ticket_id = 0
        self.log_dir = get_logs_dir()
        mock_get_logs_dir.return_value = self.log_dir
        super(TestStatus, self).setUp()

        self.assertNotEqual('/', self.log_dir)
        self.addCleanup(lambda x: shutil.rmtree(x), self.log_dir)
        # explicitly call _init_logging so that status logger is setup.
        lander._init_logging()

    @mock.patch('ci_utils.get_logs_dir')
    def test_add_status(self, mock_get_logs_dir):
        mock_get_logs_dir.return_value = self.log_dir
        data = {}
        add_status(self.ticket_id, data)

        log_file = os.path.join(self.log_dir,
                                str(self.ticket_id), LOG_STATUS_FILENAME)
        self.assertTrue(os.path.exists(log_file))

    @mock.patch('ci_utils.get_logs_dir')
    def test_log_status(self, mock_get_logs_dir):
        mock_get_logs_dir.return_value = self.log_dir
        log_status("blah")

        log_file = os.path.join(self.log_dir, STATUS_FILENAME)
        self.assertTrue(os.path.exists(log_file))

    @mock.patch('ci_utils.get_logs_dir')
    def test_add_status_verify(self, mock_get_logs_dir):
        mock_get_logs_dir.return_value = self.log_dir
        data = {'type': 'testing'}
        add_status(self.ticket_id, data)

        log_file = os.path.join(self.log_dir,
                                str(self.ticket_id), LOG_STATUS_FILENAME)

        ret_data = {}
        with open(log_file, 'r') as fp:
            ret_data = json.load(fp)

        self.assertEqual(data, ret_data)
