# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import uuid

import lander.ticket_api


class _Workflow(object):
    def __init__(self, workflow_dict):
        self.workflow = workflow_dict

    @property
    def ticket_id(self):
        return self.workflow['ticket_id']

    @property
    def ts_url(self):
        return self.workflow['ts_url']

    @property
    def progress_trigger(self):
        return self.workflow['progress_trigger']

    def save(self, fd):
        json.dump(self.workflow, fd, indent=4)

    def steps(self):
        return self.workflow['steps']

    def get_step(self, name):
        for step in self.steps():
            if name == step['name']:
                return step
        raise KeyError(name)

    def set_results(self, name, results):
        steps = self.steps()
        for i in range(len(steps)):
            if name == steps[i]['name']:
                results['name'] = name
                steps[i] = results
                return
        raise KeyError(name)

    def ticket_api(self):
        return lander.ticket_api.TicketApi(self.ts_url, self.ticket_id)


def create(ts_url, ticket_data):
    data = {
        'progress_trigger': 'lander-{}'.format(uuid.uuid4()),
        'ticket_id': ticket_data['id'],
        'ts_url': ts_url,
        'steps': ticket_data['steps'],
    }
    return _Workflow(data)


def load(fname):
    with open(fname) as f:
        data = json.load(f)
        return _Workflow(data)
