#!/usr/bin/env python3
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# import os

from wsgiref.simple_server import make_server
from pyramid.config import Configurator


from nfss.api import v1 as api_v1
from nfss.database import get_connection_for_request


def make_wsgi_app():
    config = Configurator()

    # Instead of configuring all the routes here, we defer to the individual
    # API version. This allows us to easily add 'v2' in the future. The actual
    # routes are added in the 'configure_routes' callable.
    #
    # The root app is served at '/api/v1/' - note the trailing '/'.
    config.include(api_v1.configure_routes, route_prefix='/api/v1')
    config.include(api_v1.configure_views, route_prefix='/api/v1')
    config.scan()
    try:
        import pyramid_debugtoolbar
        config.include('pyramid_debugtoolbar')
        config.registry.settings['debugtoolbar.hosts'].append('10.0.0.0/16')
    except ImportError:
        pass

    config.add_request_method(
        get_connection_for_request,
        'database',
    )
    return config.make_wsgi_app()


app = make_wsgi_app()


if __name__ == '__main__':
    server = make_server('127.0.0.1', 8000, app)
    server.serve_forever()
