# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import threading

import fixtures

from nfss.testing import fake_server


logger = logging.getLogger()


class FakeNFSSServerRunning(fixtures.Fixture):

    def __init__(self):
        super().__init__()

    def setUp(self):
        super().setUp()
        self._start_fake_server()

    def _start_fake_server(self):
        logger.info('Starting fake NFSS server.')
        server_address = ('', 0)
        self.server = fake_server.FakeNFSSServer(server_address)
        server_thread = threading.Thread(target=self.server.serve_forever)
        server_thread.start()
        logger.info('Serving at port {}.'.format(self.server.server_port))
        self.addCleanup(self._stop_fake_server, server_thread)
        self.url = 'http://localhost:{}/'.format(self.server.server_port)

    def _stop_fake_server(self, thread):
        logger.info('Stopping fake NFSS server.')
        self.server.shutdown()
        self.server.socket.close()
        thread.join()
