# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import http.client
import urllib.parse

import testtools

from nfss.testing import fixture_setup


class FakeServerRunningTestCase(testtools.TestCase):

    def setUp(self):
        super().setUp()
        self.fake_server_fixture = fixture_setup.FakeNFSSServerRunning()

    def start_fake_server(self):
        self.useFixture(self.fake_server_fixture)
        self.netloc = urllib.parse.urlparse(
            self.fake_server_fixture.url).netloc

    def assert_server_not_running(self):
        self.assertRaises(Exception, self.get_project_data, 'dummy')

    def get_project_data(self, project):
        return self.do_request('GET', '/api/v1/' + project)

    def do_request(self, method, path):
        connection = http.client.HTTPConnection(self.netloc)
        self.addCleanup(connection.close)
        connection.request(method, path)
        response = connection.getresponse()
        return response.status, json.loads(response.readall().decode('utf-8'))

    def test_server_must_start_and_stop(self):
        self.addCleanup(self.assert_server_not_running)
        self.start_fake_server()
        status, _ = self.get_project_data('dummy')
        self.assertEqual(status, 200)

    def test_get_project_data_for_unexisting_project(self):
        """Test that the data for unexisting project is empty.

        See http://pad.lv/1391033

        """
        self.start_fake_server()
        status, data = self.get_project_data('unexisting')
        self.assertEqual(status, 200)
        self.assertEqual(data, {'project_name': 'unexisting', 'tests': {}})

    def test_get_project_data_for_existing_project(self):
        """Test that the data for an existing project must have last_data."""
        self.start_fake_server()
        self.fake_server_fixture.server.add_fake_test_data(
            'test_project', 'test_test', 'test_last_data')

        status, data = self.get_project_data('test_project')
        self.assertEqual(status, 200)
        self.assertEqual(data, {
            'project_name': 'test_project',
            'tests': {
                'test_test': {
                     'last_data': 'test_last_data'
                 }
             }
         })
