'use strict';

angular.module('NonFunctional.graphing')
.controllerProvider.register("appStartupCtrl", ['$scope', '$rootScope', 'dates', 'modifiers', function($scope, $rootScope, dates, modifiers) {
    $scope.graphData = [];
    // This is a subtractor factory, that is, a function that returns a function
    // that does subtraction.
    function subtractory(minuend, subtrahend) {
        return function(item) {
            var difference = (item.data.tracepoints[0][minuend] -
                              item.data.tracepoints[0][subtrahend]) / 1000000;
            return { id: item.id, date_entered: dates.dateParser(item.date_entered), delta: difference || 0 };
        };
    }

    $rootScope.$on("newdata", function(e, data) {
        $scope.massageGraphData(data);
        $scope.api.refresh();
    });

    // Expose helpers to template.
    $scope.numberFormatter = modifiers.numberFormatter;
    $scope.dateFormatter = dates.dateFormatter;

    $scope.massageGraphData = function(blob) {
        $scope.graphData = [
            {
                key: "Callback - Sent",
                values: blob.data.map(subtractory("libual_start_message_callback",
                                                  "libual_start_message_sent"))
            },
            {
                key: "PreExec - Sent",
                values: blob.data.map(subtractory("exec_pre_exec",
                                                  "libual_start_message_sent"))
            },
            {
                key: "Callback - PreExec",
                values: blob.data.map(subtractory("libual_start_message_callback",
                                                  "exec_pre_exec"))
            }
        ];
    };

    $scope.options = {
        chart: {
            type: "lineChart",
            height: 400,
            margin: {left:100, top:10, bottom:40, right:100},
            x: modifiers.accessor('date_entered'),
            y: modifiers.accessor('delta'),
            useInteractiveGuideline: true,
            tooltips: true,
            xAxis: {
                tickFormat: dates.dateFormatter(),
                staggerLabels: true,
            },
            yAxis: {
                tickFormat: modifiers.numberFormatter(',.2f', 'ms'),
                tickPadding: 10,
            },
        }
    };
}]);
