'use strict';

angular.module('NonFunctional.graphing')
.controllerProvider.register("bootspeedCtrl", ['$scope', '$rootScope', 'dates', 'modifiers', function($scope, $rootScope, dates, modifiers) {
    $scope.graphData = [];

    $scope.dateFormatter = dates.dateFormatter;
    $scope.numberFormatter = modifiers.numberFormatter;

    $rootScope.$on("newdata", function(e, data) {
        $scope.massageGraphData(data);
        $scope.api.refresh();
    });

    function isolator(key) {
        return function(item) {
            return { date_entered: dates.dateParser(item.date_entered),
                     timespan: Math.max(item.data[key], 0) };
        };
    }

    $scope.massageGraphData = function(blob) {
        $scope.rawData = blob;
        $scope.graphData = [
            {
                key: 'Kernel',
                values: blob.data.map(isolator('kernel'))
            },
            {
                key: 'Plumbing',
                values: blob.data.map(isolator('plumbing'))
            },
            {
                key: 'XOrg',
                values: blob.data.map(isolator('xorg'))
            },
            {
                key: 'Desktop',
                values: blob.data.map(isolator('desktop'))
                //    },
                //    {
                //        key: 'Total Boot',
                //        values: blob.data.map(isolator('boot'))
            }
        ];
    };
    $scope.options = {
        chart: {
            type: "stackedAreaChart",
            height: 400,
            margin: {left:100, top:10, bottom:40, right:100},
            x: modifiers.accessor('date_entered'),
            y: modifiers.accessor('timespan'),
            useInteractiveGuideline: true,
            xAxis: {
                tickFormat: dates.dateFormatter(),
                staggerLabels: true,
            },
            yAxis: {
                tickFormat: modifiers.numberFormatter(',.2f', 's'),
                tickPadding: 10,
            },
        },
    };
}]);
