'use strict';

// Graphing service module that provides the mechanisms to load and change
// graphs.

var graphing_module = angular.module('NonFunctional.graphing', []);

graphing_module.config(['$controllerProvider', function($controllerProvider) {
    // Store for later use within graph definitions.
    graphing_module.controllerProvider = $controllerProvider;
}]);

graphing_module.provider('definitions', function() {
    this.definitions = {
        // An entry per graph.
        // 'graphName': {
        //     'templateUrl': 'graphing/graphTemplate.html',
        //     'deps': 'graphing/graphTemplate.js'
        // },
        // Provide a default graph def.
        'default': {
            'templateUrl': 'graphs/default.html',
            'deps': 'graphs/default.js'
        },
        'lrt': {
            'templateUrl': 'graphs/lrt.html',
            'deps': 'graphs/lrt.js'
        },
        'lrt_ap_core_apps': {
            'templateUrl': 'graphs/lrt_ap_core_apps.html',
            'deps': 'graphs/lrt.js'
        },
        'lrt_random_gestures': {
            'templateUrl': 'graphs/lrt_random_gestures.html',
            'deps': 'graphs/lrt.js'
        },
         'lrt_switch': {
            'templateUrl': 'graphs/lrt_switch.html',
            'deps': 'graphs/lrt.js'
        },
        'memevent': {
            'templateUrl': 'graphs/memevent.html',
            'deps': 'graphs/memevent.js'
        },
        'bootspeed': {
            'templateUrl': 'graphs/bootspeed.html',
            'deps': 'graphs/bootspeed.js'
        },
        'app-startup': {
            'templateUrl': 'graphs/app-startup.html',
            'deps': 'graphs/app-startup.js'
        },
    };

    this.getGraphDef = function(url_object) {
        var definition_name = this.getGraphName(url_object);
        return this.definitions[definition_name];
    };

    this.getSlug = function(url_object) {
        // returns a value slug value of the project/test provided, undefined
        // otherwise.
        if (url_object.project && url_object.test) {
            return url_object.project + '_' + url_object.test;
        }
        return undefined;
    };

    this.getGraphName = function(url_object) {
        // Determine which graph name has been requested, defaults to 'default'
        // if unknown.
        var slug = this.getSlug(url_object);
        if(this.definitions[slug]) {
            return slug;
        } else if(this.definitions[url_object.project]) {
            return url_object.project;
        } else if(this.definitions[url_object.test]) {
            return url_object.test;
        } else {
            return 'default';
        }
    };

    this.$get = function() {
        return {
            getGraphDef: this.getGraphDef,
            definitions: this.definitions,

            getGraphName: this.getGraphName,
            getSlug: this.getSlug
        };
    };
});
