'use strict';

angular.module('NonFunctional.graphing')
.controllerProvider.register("lrtCtrl", ['$scope', '$rootScope', 'dates', 'modifiers', function($scope, $rootScope, dates, modifiers) {
    $scope.tableDataFormStrings = [ ',.2f', ',.2f', ',.2f', ',.2f', ',d' ];
    $scope.tableDataUnitStrings = [ 'm', 'm', 'm', 'm', '' ];

    function isolator(key, factor) {
        return function(item) {
            return {
                value: Math.max(item.data[key], 0) / (factor || 1),
                image: item.data['image'],
                date: dates.dateParser(item.date_entered),
            };
        };
    }

    // Expose helper to the template.
    $scope.numberFormatter = modifiers.numberFormatter;

    $scope.massageGraphData = function(blob) {
        $scope.rawData = blob;

        $scope.graphData = [
            {
                key: 'Average Minutes to Crash',
                bar: true,
                values: blob.data.map(isolator('avg', 60)),
            },
            {
                key: 'Standard Deviation',
                values: blob.data.map(isolator('stddev', 60)),
            },
        ];

        // Figure out the largest value we show on the graph - we use this to set the
        // bar and line y axis values.
        $scope.maxY = Math.max.apply(null, $scope.graphData.map(function(data_series) {
            return d3.max(data_series.values, function(i) {return i.value;});
        }));

        $scope.tableData = $scope.graphData.concat([
            {
                key: 'Min',
                values: blob.data.map(isolator('min', 60)),
            },
            {
                key: 'Max',
                values: blob.data.map(isolator('max', 60)),
            },
            {
                key: 'Number of Samples',
                values: blob.data.map(isolator('num_samples')),
            }
        ]);
    };

    $rootScope.$on("newdata", function(e, data) {
        $scope.massageGraphData(data);
        $scope.options.chart.lines = { forceY: [0, $scope.maxY] };
        $scope.options.chart.bars = { forceY: [0, $scope.maxY] };
        $scope.api.refresh();
    });

    $scope.options = {
        chart: {
            type: 'linePlusBarChart',
            height: 400,
            margin: {left:100, top:10, bottom:120, right:100},
            // X axis values are just the indexes...
            x: function(item, idx) { return idx; },
            y: modifiers.accessor('value'),
            useInteractiveGuideline: true,
            xAxis: {
                // and we translate those to image numbers here:
                tickFormat: function(idx) { 
                    if($scope.graphData[0].values[idx]) {
                        // make this end in '.date' if you'd rather show the date.
                        return $scope.graphData[0].values[idx].image;  
                    } 
                },
                rotateLabels: -45,
            },
            y1Axis: {
                tickFormat: modifiers.numberFormatter(',.2f', ' min'),
            },
            y2Axis: {
                tickFormat: modifiers.numberFormatter(',.2f', ' min'),
            },
        }
    };
}]);
