'use strict';

angular.module('NonFunctional.graphing')
.controllerProvider.register("memeventCtrl", ['$scope', '$rootScope', 'dates', 'modifiers', function($scope, $rootScope, dates, modifiers) {
    $scope.graphData = [];
    function namer(key, event) {
        return key + ' ' + event;
    }

    $rootScope.$on("newdata", function(e, data) {
        $scope.massageGraphData(data);
        $scope.api.refresh();
    });

    // Expose these to the template
    $scope.numberFormatter = modifiers.numberFormatter;
    $scope.dateFormatter = dates.dateFormatter;
    $scope.ISO_ISH = dates.ISO_ISH;

    $scope.massageGraphData = function(blob) {
        // Clear existing data.
        $scope.graphData = [];
        var eventMap = {};
        var first_with_data = 0;

        blob.data.some(function(datum, index) {
            first_with_data = index;
            return datum.data.events.length;
        });

        [ "RSS", "PSS", "USS", "Swap" ].forEach(function(key) {
            blob.data[first_with_data].data.events.forEach(function(event) {
                var name = namer(key, event.event);
                eventMap[name] = $scope.graphData.length;
                $scope.graphData.push({
                    key: name,
                    values: [],
                });
            });

            blob.data.forEach(function(datum) {
                datum.data.events.forEach(function(event) {
                    $scope.graphData[eventMap[namer(key, event.event)]].values.push({
                        mem_usage: event.data[key.toLowerCase()],
                        date_entered: dates.dateParser(datum.date_entered),
                    });
                });
            });
        });

    };

    $scope.options = {
        chart: {
            type: 'lineChart',
            height: 400,
            margin: {left:100, top:10, bottom:40, right:100},
            x: modifiers.accessor('date_entered'),
            y: modifiers.accessor('mem_usage'),
            useInteractiveGuideline: true,
            xAxis: {
                tickFormat: dates.dateFormatter(),
                staggerLabels: true,
            },
            yAxis: {
                tickFormat: modifiers.numberFormatter(',f', 'kiB'),
            },
        }
    };
}]);
