'use strict';

angular.module('NonFunctional.support', [])
.factory('dates', function() {
    var ISO_EIGHT_SIX_OH_ONE = "%Y-%m-%dT%H:%M:%SZ";
    var ISO_ISH = "%Y-%m-%d %H:%M:%S %Z";

    function dateParser(dateString) {
        var parts = dateString.split(/[-:.+ TZ]/g);
        return new Date(Date.UTC(parts[0], parts[1] - 1, parts[2],
                                 parts[3], parts[4], parts[5]));
    }

    // Display dates nicely. For information on the format string, refer to
    // https://github.com/mbostock/d3/wiki/Time-Formatting#format
    function dateFormatter(formatString) {
        return function(date) {
            return d3.time.format(formatString || ISO_ISH)(new Date(date));
        };
    }

    // Create a new date object that is an arbitrary number of months offset
    // from the passed-in date object.
    function deltaMonth(date, months) {
        date = new Date(date);
        date.setMonth(date.getMonth() + months);
        return date;
    }

    return {
        ISO_EIGHT_SIX_OH_ONE: ISO_EIGHT_SIX_OH_ONE,
        ISO_ISH: ISO_ISH,
        dateParser: dateParser,
        dateFormatter: dateFormatter,
        deltaMonth: deltaMonth,
    };
})
.factory('modifiers', function() {
    // Display numbers nicely. For information on the format string, refer to
    // https://github.com/mbostock/d3/wiki/Formatting#d3_format
    function numberFormatter(formatString, units) {
        return function(number) {
            return (d3.format(formatString || ',.2f')(number) + (units || ''));
        };
    };

    // 'item' refers to each entry in the blob.data array, and allows for
    // graph definitions that look like y="accessor('delta')"
    function accessor(key) { return function(item) { return item[key]; }; };

    return {
        numberFormatter:numberFormatter,
        accessor: accessor,
    };
});
