'use strict';

describe("graphing module", function() {
    var definitions;
    beforeEach(module("NonFunctional.graphing"));
    beforeEach(inject(function(_definitions_) {
        definitions = _definitions_;
    }));

    describe("definitions", function() {
        beforeEach(function() {
            // Define a known graph definitions dict.
            definitions.definitions = {
                'known_slug': true,
                'project': true,
                'test': true,
                'default': 'default',
            };
        });

        describe('getSlug function', function(){
            it('must return undefined with no project or test in url', function() {
                expect(definitions.getSlug({})).toBeUndefined();
            });

            it('must return undefined with just project defined no test in url', function() {
                expect(definitions.getSlug({project:''})).toBeUndefined();
            });

            it('must return undefined with test defined but no project in url', function() {
                expect(definitions.getSlug({test:''})).toBeUndefined();
            });

            it('must return combined project_test slug when both defined', function() {
                expect(
                    definitions.getSlug({project: 'project',  test:'test'})
                ).toBe('project_test');
            });
        });

        describe("getGraphName function", function() {
            it('must return \'default\' with unknown test name', function() {
                expect(definitions.getGraphName({})).toBe('default');
            });

            it('must return project name with known project in url', function() {
                expect(
                    definitions.getGraphName({project: 'project'})
                ).toBe('project');
            });

            it('must return test name with known test in url', function() {
                expect(definitions.getGraphName({test: 'test'})).toBe('test');
            });

            it('must return \'project_test\' slug name with known test and project in url', function() {
                expect(
                    definitions.getGraphName({project: 'known', test: 'slug'})
                ).toBe('known_slug');
            });

            it('must return project name with known project but unknown test in url', function() {
                expect(
                    definitions.getGraphName({project: 'project', test: 'nothing'})
                ).toBe('project');
            });

            it('must return test name with known test but unknown project in url', function() {
                expect(
                    definitions.getGraphName({project: 'unknown', test: 'test'})
                ).toBe('test');
            });
        });

        describe("getGraphDef function", function() {
            it('must return default graph def for unknown', function() {
                expect(definitions.getGraphDef({project: ''})).toBe('default');
            });

            it('must return correct known graph def for unknown', function() {
                expect(definitions.getGraphDef({project: 'project'})).toBe(true);
            });
        });
    });
});
