#!/usr/bin/env python
# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Check ImageStore features in an existing deployment."""
import os
import tempfile
import unittest

from ci_utils import unit_config
from ci_utils.image_store import ImageStore


class ImageStoreTest(unittest.TestCase):

    def test_image_creation(self):
        # `ImageStore.image_create` is capable of uploading an arbitrary
        # image to the available Glance server. This is how the Image-Builder
        # publishes a testing image to the Test-Runner.
        _fd, image_path = tempfile.mkstemp()
        self.addCleanup(os.remove, image_path)
        open(image_path, 'w').write('I can haz image ...')
        config = unit_config.get_auth_config()

        glance = ImageStore(config)
        image = glance.image_create('ci-airline-testing.img', image_path)
        self.addCleanup(glance.image_delete, image.id)
        self.assertEquals('active', image.status)


if __name__ == "__main__":
    unittest.main()
