#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import sys


HERE = os.path.abspath(os.path.dirname(__file__))
# get ci_utils.setuptools on the path
sys.path.append(os.path.join(HERE, '..', 'ci-utils'))
from ci_utils.setup_helper import setup


if sys.argv[1] == 'create_db':
    import ticket_system.settings
    import ci_utils.create_db
    dbconfig = ticket_system.settings.DATABASES['default']
    sys.exit(ci_utils.create_db.create_db(dbconfig))
elif sys.argv[1] == 'test':
    # setuptools doesn't work with django's test harness very well, so
    # we'll just run everything by hand from here
    args = ['./manage.py', 'test']
    if len(sys.argv) > 2:
        args += sys.argv[2:]
    else:
        from ticket_system import settings
        # Default to local apps
        args.extend(settings.LOCAL_APPS)
    sys.exit(subprocess.call(args, cwd=HERE))


requires = [
    'Django==1.5.5',
    'South==0.7.5',
    'amqplib==1.0.0',
    'django-tastypie==0.9.15',
    'mock==1.0.1',
    'model_mommy>=1.2',
    'lazr.enum>=1.1.2',
    'PyYAML==3.10',
    'django-oauth2-provider>=0.2.6.1',
    'python-openid==2.2.5',
    'django-openid-auth==0.2',
]


extra = {
    'requires_openstack': True,
}


setup('ticket_system', 'ticket_system.local_tests', requires, **extra)
