# Ubuntu Continuous Integration Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from ticket.models import (
    MergeProposal,
    Review,
    SourcePackageUpload,
    SubTicket,
    SubTicketArtifact,
    Ticket,
    TicketArtifact,
    Workflow,
)


class TicketAdmin(admin.ModelAdmin):
    list_filter = ['current_workflow_step', 'status']
    search_fields = ['title', 'bug_id', 'uuid']
    list_display = ('title', 'current_workflow_step', 'status', 'base_image')
    readonly_fields = ('uuid',)


class SubTicketAdmin(admin.ModelAdmin):
    list_filter = ['status']
    search_fields = ['ticket']
    list_display = ('ticket', 'sourcepackage', 'status', 'assignee',
                    'source_package_uploads')


class SourcePackageUploadAdmin(admin.ModelAdmin):
    search_fields = ['subticket__sourcepackage']
    list_display = ('subticket', 'version', 'architecture')


class TicketArtifactAdmin(admin.ModelAdmin):
    list_filter = ['type']
    search_fields = ['name']
    list_display = ('name', 'ticket', 'type')


class SubTicketArtifactAdmin(admin.ModelAdmin):
    list_filter = ['type']
    search_fields = ['name']
    list_display = ('name', 'subticket', 'type')


class WorkflowAdmin(admin.ModelAdmin):
    list_filter = ['name']
    search_fields = ['name']


admin.site.register(MergeProposal)
admin.site.register(Review)
admin.site.register(SourcePackageUpload, SourcePackageUploadAdmin)
admin.site.register(SubTicket, SubTicketAdmin)
admin.site.register(SubTicketArtifact, SubTicketArtifactAdmin)
admin.site.register(Ticket, TicketAdmin)
admin.site.register(TicketArtifact, TicketArtifactAdmin)
admin.site.register(Workflow, WorkflowAdmin)
