# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile

from django.test import TestCase
from django.core.management import call_command, CommandError


class TestCreatClient(TestCase):

    def _call_create_client(self, redirect_uri):

        fd, filepath = tempfile.mkstemp()
        output = ""
        with os.fdopen(fd, 'rb+') as f:
            call_command('create_client', redirect_uri, stdout=f)
            f.seek(0)
            output = f.read()
            self.assertIn(':', output)

        return output

    def test_create_client_success(self):
        output = self._call_create_client('http://example.com/')
        self.assert_(output)

    def test_create_client_again_success(self):
        """Test that a second call with the same redirect_uri returns the
        same data not new data."""

        output = self._call_create_client('http://example.com/')
        second_output = self._call_create_client('http://example.com/')

        self.assertEqual(output, second_output)

    def test_requires_argument_none(self):
        with self.assertRaises(CommandError):
            output = self._call_create_client(None)
            self.assert_(output)

    def test_requires_argument_blank(self):
        with self.assertRaises(CommandError):
            output = self._call_create_client("")
            self.assert_(output)
