# Ubuntu Continuous Integration Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from django.conf import settings
from model_mommy import mommy
from model_mommy.recipe import Recipe, seq

from ci_utils.tastypie.test import TicketTastypieTestCase
from project.models import SourcePackage
from ticket.models import (
    get_enum_title,
    SubTicketWorkflowStep,
    SubTicketWorkflowStepStatus,
    TicketWorkflowStep,
    TicketWorkflowStepStatus,
)


sourcepackage_recipe = Recipe(SourcePackage, name=seq('foobar'))


class TicketFullReadAPITest(TicketTastypieTestCase):

    def setUp(self):
        super(TicketFullReadAPITest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.sourcepackage = sourcepackage_recipe.make()
        self.subticket = mommy.make(
            'SubTicket', ticket=self.ticket, sourcepackage=self.sourcepackage)
        self.spu = mommy.make(
            'SourcePackageUpload', subticket=self.subticket)
        self.artifact_1 = mommy.make('TicketArtifact', ticket=self.ticket)
        self.artifact_2 = mommy.make(
            'SubTicketArtifact', subticket=self.subticket)

    def test_lookup_by_id(self):
        # A `FullTicket` can be retrieved using its ID (pk, integer).
        obj = self.getResource('fullticket/{}/'.format(self.ticket.id))
        self.assertEqual(self.ticket.id, obj['id'])

    def test_lookup_by_uuid(self):
        # A `FullTicket` can be also retrieved by its UUID when it's set.
        obj = self.getResource('fullticket/{}/'.format(self.ticket.uuid))
        self.assertEqual(self.ticket.id, obj['id'])

    def test_get_fullticket_api(self):
        obj = self.getResource('fullticket/')
        self.assertEqual(obj['objects'][0], {
            u'artifact': [{
                u'resource_uri': u'/api/v1/fullticketartifact/{0}/'.format(
                    self.artifact_1.pk),
                u'type': unicode(self.artifact_1.type),
                u'id': self.artifact_1.pk,
                u'reference': unicode(self.artifact_1.reference),
                u'name': unicode(self.artifact_1.name)}],
            u'status': unicode(get_enum_title(self.ticket.status,
                               TicketWorkflowStepStatus)),
            u'current_workflow_step': unicode(get_enum_title(
                self.ticket.current_workflow_step, TicketWorkflowStep)),
            u'description': unicode(self.ticket.description),
            u'title': unicode(self.ticket.title),
            u'private': False,
            u'subticket': [{
                u'status': unicode(get_enum_title(self.subticket.status,
                                   SubTicketWorkflowStepStatus)),
                u'current_workflow_step': unicode(get_enum_title(
                    self.subticket.current_workflow_step,
                    SubTicketWorkflowStep)),
                u'artifact': [{
                    u'resource_uri':
                    u'/api/v1/fullsubticketartifact/{0}/'.format(
                        self.artifact_2.pk),
                    u'type': unicode(self.artifact_2.type),
                    u'id': self.artifact_2.pk,
                    u'reference': unicode(self.artifact_2.reference),
                    u'name': unicode(self.artifact_2.name)}],
                u'assignee': unicode(self.subticket.assignee),
                'sourcepackage': {
                    'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                        self.sourcepackage.pk),
                    'id': self.sourcepackage.pk,
                    'name': unicode(self.sourcepackage.name)},
                'merge_proposals': [],
                'source_package_uploads': [{
                    'architecture': unicode(self.spu.architecture),
                    'version': unicode(self.spu.version),
                    'id': self.spu.pk,
                    'resource_uri': '/api/v1/spu/{0}/'.format(self.spu.pk),
                    'subticket': '/api/v1/subticket/{0}/'.format(
                        self.subticket.pk)}],
                u'id': self.subticket.pk,
                u'conflicts': [],
                u'resource_uri': u'/api/v1/fullsubticket/{0}/'.format(
                    self.subticket.pk)}],
            u'bug_id': self.ticket.bug_id,
            u'created': unicode(self.ticket.created.strftime(
                settings.TEST_DATETIME_FORMAT)),
            u'owner': unicode(self.ticket.owner),
            u'creator': unicode(self.ticket.creator),
            u'base_image': unicode(self.ticket.base_image),
            u'id': self.ticket.pk,
            u'lander_unit': 0,
            u'master_ppa': unicode(self.ticket.master_ppa),
            u'build_ppa': unicode(self.ticket.build_ppa),
            u'resource_uri': u'/api/v1/fullticket/{0}/'.format(
                self.ticket.uuid),
            u'series': unicode(self.ticket.series),
            u'updated': unicode(self.ticket.updated.strftime(
                settings.TEST_DATETIME_FORMAT)),
            u'uuid': self.ticket.uuid,
            u'reviews': [],
            u'citrain_overlay': None,
            'workflow': {
                'name': 'default',
                'steps': [
                    {'name': 'ppa_creator'},
                    {'name': 'bsbuilder'},
                    {'name': 'validator'},
                    {'name': 'image_builder'},
                    {'name': 'test_runner'},
                    {'name': 'publisher'},
                ]},
        })

    def test_get_fullticketartifact_api(self):
        obj = self.getResource('fullticketartifact/')
        self.assertEqual(obj['objects'][0], {
            u'resource_uri': u'/api/v1/fullticketartifact/{0}/'.format(
                self.artifact_1.pk),
            u'type': unicode(self.artifact_1.type),
            u'id': self.artifact_1.pk,
            u'reference': unicode(self.artifact_1.reference),
            u'name': unicode(self.artifact_1.name),
        })

    def test_get_fullsubticketartifact_api(self):
        obj = self.getResource('fullsubticketartifact/')
        self.assertEqual(obj['objects'][0], {
            u'resource_uri': u'/api/v1/fullsubticketartifact/{0}/'.format(
                self.artifact_2.pk),
            u'type': unicode(self.artifact_2.type),
            u'id': self.artifact_2.pk,
            u'reference': unicode(self.artifact_2.reference),
            u'name': unicode(self.artifact_2.name),
        })

    def test_get_fullsubticket_api(self):
        obj = self.getResource('fullsubticket/')
        self.assertEqual(obj['objects'][0], {
            u'status': unicode(get_enum_title(self.subticket.status,
                                              SubTicketWorkflowStepStatus)),
            u'current_workflow_step': unicode(get_enum_title(
                self.subticket.current_workflow_step,
                SubTicketWorkflowStep)),
            u'artifact': [{
                u'resource_uri': u'/api/v1/fullsubticketartifact/{0}/'.format(
                    self.artifact_2.pk),
                u'type': unicode(self.artifact_2.type),
                u'id': self.artifact_2.pk,
                u'reference': unicode(self.artifact_2.reference),
                u'name': unicode(self.artifact_2.name)}],
            u'assignee': unicode(self.subticket.assignee),
            'sourcepackage': {
                'resource_uri': u'/api/v1/sourcepackage/{0}/'.format(
                    self.sourcepackage.pk),
                'id': self.sourcepackage.pk,
                'name': unicode(self.sourcepackage.name)},
            'merge_proposals': [],
            'source_package_uploads': [{
                'architecture': unicode(self.spu.architecture),
                'version': unicode(self.spu.version),
                'id': self.spu.pk,
                'resource_uri': '/api/v1/spu/{0}/'.format(self.spu.pk),
                'subticket': '/api/v1/subticket/{0}/'.format(
                    self.subticket.pk)}],
            u'id': self.subticket.pk,
            u'conflicts': [],
            u'resource_uri': u'/api/v1/fullsubticket/{0}/'.format(
                self.subticket.pk),
        })
