# Ubuntu Continuous Integration Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function

import json
import os
import sys
import yaml

HERE = os.path.abspath(os.path.dirname(__file__))
BASEDIR = os.path.abspath(os.path.join(HERE, '..'))

DEBUG = False
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS


def _unit_config():
    path = os.path.join(BASEDIR, '../../../unit_config')
    config = {}
    try:
        with open(path) as f:
            config = yaml.safe_load(f.read())
    except:
        print('Unable to use unit_config(%s), defaulting values' % path,
             file=sys.stderr)
    return config
_cfg = _unit_config()

SERIES_DEFAULT = _cfg.get('series', '')
BASE_IMAGE_MAP = _cfg.get('image_map')
if type(BASE_IMAGE_MAP) == dict:
    BASE_IMAGE_DEFAULT = BASE_IMAGE_MAP.get(SERIES_DEFAULT, '')
else:
    BASE_IMAGE_DEFAULT = ''
MASTER_PPA_DEFAULT = _cfg.get('master_ppa', '')

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASEDIR, 'ticket_system.sqlite'),
    }
}

# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = []

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'GMT'
TASTYPIE_DATETIME_FORMATTING = 'rfc-2822'
TEST_DATETIME_FORMAT = "%a, %-d %b %Y %H:%M:%S %z"

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    #'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '1)*_ck0b)6mek=&dud8(g(=13g$s)_bwu#vzsq0g6yfjgdx-zb'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    # 'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'ticket_system.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'ticket_system.wsgi.application'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or
    # "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django_openid_auth',
    'south',
    'tastypie',
    'provider',
    'provider.oauth2',
)

LOCAL_APPS = (
    #'people',
    'project',
    'ticket',
)

INSTALLED_APPS = LOCAL_APPS + INSTALLED_APPS
TEST_RUNNER = "ticket_system.local_tests.LocalAppsTestSuiteRunner"
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.JSONSerializer'
TASTYPIE_DEFAULT_FORMATS = ['json']

SESSION_SERIALIZER = 'django.contrib.sessions.serializers.JSONSerializer'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

OPENID_CREATE_USERS = True
LOGIN_URL = '/openid/login/'
LOGIN_REDIRECT_URL = '/api/v1/'
OPENID_SSO_SERVER_URL = 'https://login.ubuntu.com/'
AUTHENTICATION_BACKENDS = [
    'django_openid_auth.auth.OpenIDBackend',
    'django.contrib.auth.backends.ModelBackend',
]
SESSION_SERIALIZER = 'django.contrib.sessions.serializers.PickleSerializer'

# Use local settings if available
try:
    p = os.path.join(BASEDIR, 'local_settings.py')
    if os.path.exists(p):
        execfile(p)
except ImportError:
    pass


# files created by restish juju charm
pgsql = os.path.join(BASEDIR, '../../../pgsql.json')
if os.path.exists(pgsql):
    with open(pgsql) as f:
        pgsql = json.load(f)
        DATABASES['default'] = {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': pgsql['database'],
            'USER': pgsql['schema_user'],
            'PASSWORD': pgsql['schema_password'],
            'HOST': pgsql['host'],
            'PORT': pgsql['port'],
        }

allowed = os.path.join(BASEDIR, '../../../allowed_hosts.json')
if os.path.exists(allowed):
    with open(allowed) as f:
        ALLOWED_HOSTS = json.load(f)

logging = os.path.join(BASEDIR, '../../../logging.json')
if os.path.exists(logging):
    data = {}
    with open(logging) as f:
        data = json.load(f)

    log_dir = data.get('log_dir')
    log_level = data.get('log_level')

    if log_dir and os.path.exists(log_dir) and log_level:
        log_file = os.path.join(log_dir, 'django.log')
        LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': {
                'standard': {
                    'format': (
                        "[%(asctime)s] %(levelname)s [%(name)s"
                        ":%(lineno)s] %(message)s"),
                    'datefmt': "%d/%b/%Y %H:%M:%S"
                },
            },
            'handlers': {
                'file': {
                    'level': log_level,
                    'class': 'logging.FileHandler',
                    'filename': log_file,
                    'formatter': 'standard',
                },
            },
            'loggers': {
                'django': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
                '': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
            },
        }
