# Ubuntu Continuous Integration Engine
# Copyright 2013, 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import (
    include,
    patterns,
    url
)
from django.contrib import admin
from django.contrib.auth.views import logout as django_logout
from tastypie.api import Api

from people import api as people_api
from project import api as project_api
from ticket import api as ticket_api


def local_logout(request):
    """Forces user logout.

    Also redirects to '/' (root) if nothing else ('?next=...') was
    specified by the requester.
    """
    return django_logout(request, next_page='/')


admin.autodiscover()
v1_api = Api(api_name='v1')
v1_api.register(people_api.PersonResource())
v1_api.register(project_api.SourcePackageResource())
v1_api.register(project_api.BinaryPackageResource())
v1_api.register(ticket_api.TicketResource())
v1_api.register(ticket_api.SubTicketResource())
v1_api.register(ticket_api.SourcePackageUploadResource())
v1_api.register(ticket_api.MergeProposalResource())
v1_api.register(ticket_api.TicketArtifactResource())
v1_api.register(ticket_api.SubTicketArtifactResource())
v1_api.register(ticket_api.FullTicketArtifactResource())
v1_api.register(ticket_api.FullSubTicketArtifactResource())
v1_api.register(ticket_api.FullSubTicketResource())
v1_api.register(ticket_api.FullTicketResource())
v1_api.register(ticket_api.SubTicketUpdateStatusResource())
v1_api.register(ticket_api.SubTicketStatusResource())
v1_api.register(ticket_api.TicketWorkflowStepResource())
v1_api.register(ticket_api.SubTicketWorkflowStepResource())
v1_api.register(ticket_api.TicketWorkflowStepStatusResource())
v1_api.register(ticket_api.SubTicketWorkflowStepStatusResource())
v1_api.register(ticket_api.ReviewResource())
v1_api.register(ticket_api.TSStatus())
v1_api.register(ticket_api.UserInfoResource())


urlpatterns = patterns(
    '',
    (r'^api/', include(v1_api.urls)),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^oauth2/', include('provider.oauth2.urls', namespace='oauth2')),
    url(r'^openid/', include('django_openid_auth.urls')),
    url(r'^logout/$', local_logout, name='logout'),
)
