# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


class ValidatorProcessError(Exception):
    """Raised when there is an error processing validation request."""


def validate_subticket(subticket, series, source_archive,
                       destination_archive):
    """Validates `Subticket` information."""
    # Until the consistency checks are added, validate should always return
    # success.
    return True


def process_request(params, data_store, status_cb=None):
    """Process a validation request."""
    try:
        # Required variables
        subtickets = params['subtickets']
        series = params['series']
        source_archive = params['source_archive']
        destination_archive = params['destination_archive']
    except KeyError as exc:
        raise ValidatorProcessError(
            "Missing required variables: '{}'.".format(exc.message))

    # Register a no-op status-callback if it wasn't provided.
    if status_cb is None:
        status_cb = lambda msg: None

    packages = []
    out_data = {
        'checks': [],
    }
    # Validade all subtickets.
    for subticket in subtickets:
        package_name = subticket['sourcepackage']['name']
        subticket_results = {
            'subticket_id': subticket['id'],
            'package': package_name,
            'result': 'FAILED',
            'info': '',
        }
        try:
            validate_subticket(subticket, series, source_archive,
                               destination_archive)
        except ValidatorProcessError as exc:
            subticket_results['result'] = 'FAILED'
            subticket_results['info'] = exc.message
            status_cb(
                'Validating source \'{}\': FAILED\nERROR: {}'.format(
                    package_name, exc.message))
        else:
            subticket_results['result'] = 'PASSED'
            status_cb(
                'Validating source \'{}\': PASSED'.format(package_name))

        packages.append(package_name)
        out_data['checks'].append(subticket_results)

    return out_data
