#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2014 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging


from ci_utils import (
    amqp_utils,
    amqp_worker,
)
from validator import (
    process_request,
    ValidatorProcessError,
)


class ValidatorWorker(amqp_worker.AMQPWorker):

    def handle_request(self, params, log):
        # Setup the output data to send back to the caller
        out_data = {}
        try:
            ticket_id = params['ticket_id']
            progress_trigger = params['progress_trigger']
        except KeyError as exc:
            out_data['status'] = 'FAILED'
            out_data['message'] = 'Invalid input: {}'.format(exc.message)
            return amqp_utils.progress_failed, out_data

        def status_cb(msg):
            log.info(msg)
            amqp_utils.progress_update(progress_trigger, {'message': msg})

        status_cb('Processing ticket {}:'.format(ticket_id))

        ds = self._create_data_store('ticket-{}'.format(ticket_id))

        try:
            result_data = process_request(params, ds, status_cb)
        except ValidatorProcessError as exc:
            out_data['status'] = 'FAILED'
            out_data['message'] = exc.message
            return amqp_utils.progress_failed, out_data

        out_data.update(result_data)

        return amqp_utils.progress_completed, out_data


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format="%(asctime)s %(levelname)s %(message)s")
    ValidatorWorker('validator').main(amqp_utils.VALIDATOR_QUEUE)
