import json
import urllib2

from django.http import HttpResponse, HttpResponseServerError
from django.shortcuts import render_to_response

from ci_utils import amqp_utils


def _get(url, realm=None, user=None, password=None):
    if realm:
        mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
        mgr.add_password(None, realm, user, password)
        handler = urllib2.HTTPBasicAuthHandler(mgr)

        resp = urllib2.build_opener(handler).open(url)
        return resp.read()

    return urllib2.urlopen(url)


def _queue_health():
    amqp_config = amqp_utils.get_config()
    queues = amqp_utils.MONITORED_QUEUES
    base = 'http://{}:15672'.format(amqp_config.AMQP_HOST)

    try:
        content = _get(
            base + '/api/queues', base,
            amqp_config.AMQP_USER, amqp_config.AMQP_PASSWORD)
        data = json.loads(content)
        data = [x for x in data if x['name'] in queues]
        # find missing queues
        found = [x['name'] for x in data]
        for x in queues:
            if x not in found:
                data.append({'name': x, 'consumers': None})
        return data
    except Exception as e:
        return {'error': str(e)}


def index(request):
    return render_to_response('status/index.html', {'active': 'status'})


def queue_health(request):
    health = _queue_health()
    return HttpResponse(json.dumps(health), content_type='application/json')


def _find_json_health_errors():
    errors = []
    try:
        resp = json.loads(_get('/static/engine-health/status_urls.json'))
        for node in resp.values():
            try:
                resp = json.loads(_get(node['url']))
                for check in resp:
                    if check.get('status', 'fail') != 'okay':
                        errors.append('{label}: {value}'.format(**check))
            except:
                errors.append('no status for: {}'.format(node['unit']))
    except:
        errors.append('Unable to check json health status')
    return errors


def _find_worker_errors():
    errors = []
    for q in _queue_health():
        if q['consumers'] in (None, 0):
            errors.append('Missing worker for: {}'.format(q['name']))
    return errors


def health_check(request):
    errors = _find_worker_errors()
    errors += _find_json_health_errors()

    if len(errors):
        errors = '\n'.join(errors)
        return HttpResponseServerError(errors, content_type='plain/text')
    return HttpResponse('OK', content_type='plain/text')
