"use strict";

// Make jslint happy
var YUI;

YUI.add('webui', function (Y) {

    /**
     * InlineReview widget.
     *
     * Allows reviews to be edited inline via TS JSON-XHR.
     *
     */
    var InlineReview = function () {
        InlineReview.superclass.constructor.apply(this, arguments);
    };

    Y.mix(InlineReview, {

        NAME: 'inlinereview',

        ATTRS: {

            review: {
                value: {}
            },

            readonly: {
                value: false
            }

        }

    });

    Y.extend(InlineReview, Y.Widget, {

        renderUI: function() {
            var review = this.get('review');

            // Empty previous content.
            this.get('srcNode').empty();

            // Review label.
            var label = Y.Node.create('<label>')
                .set('text', review.review_type)
                .addClass(this.get('review').status.toLowerCase());
            this.get('srcNode').appendChild(label);

            // Only render the review label if the widget is readonly.
            if (this.get('readonly')) { return; }

            // Link that will render the review overlay options.
            var overlay_link = Y.Node.create('<a>')
                .set('id', 'review-link-' + review.id)
                .set('href', '#')
                .addClass('edit-link')
                .set('title', 'Review');
            this.get('srcNode').appendChild(overlay_link);

            // Overlay content with review options as links.
            var overlay_content = Y.Node.create('<div>')
                .set('id', 'review-options-' + review.id)
                .addClass('hidden');
            var statuses = ['Approved', 'Disapproved', 'Pending'];
            statuses.forEach(function (s) {
                var choice = Y.Node.create('<a>')
                    .set('text', s)
                    .set('href', '#')
                    .addClass(s.toLowerCase())
                    .addClass('review-choice')
                if (s === review.status) {
                    choice.addClass('current');
                }
                overlay_content.appendChild(choice);
                overlay_content.appendChild('<hr>');
            });
            this.get('srcNode').appendChild(overlay_content);

            // Overlay widget.
            Y.Base.mix(Y.Overlay, [Y.WidgetAutohide]);
            var overlay = new Y.Overlay({
                srcNode: '#review-options-' + review.id,
                align: {
                    node:'#review-link-' + review.id,
                    points: [
                        Y.WidgetPositionAlign.TL,
                        Y.WidgetPositionAlign.BR
                    ]
                },
                hideOn: [{eventName: 'clickoutside'}],
                visible: false,
                render: true
            });

            // Hook the overlay link.
            overlay_link.on('click', function (e) {
                e.preventDefault();
                overlay.show();
            });

        },

        bindUI: function() {
            var review = this.get('review');
            var trigger = this.get('srcNode').all('a.review-choice');
            var self = this;
            trigger.on('click', function(e) {
                e.halt();
                review.status = e.currentTarget.get('text');
                var data = {
                    reviews: [review]
                };
                // Issue a PATCH TS-request to update the review and
                // update the widget accordingly.
                Y.io(review.ticket, {
                    method: 'patch',
                    data: Y.JSON.stringify(data),
                    headers: {
                        'Content-Type': 'application/json;',
                    },
                    on: {
                        start: function () {
                            self.get('srcNode').all('.error').remove(true);
                        },
                        success: function(tx, r) {
                            self.renderUI();
                            self.bindUI();
                        },
                        failure: function (tx, r) {
                            Y.log('FAILURE: ' + r.responseText);
                            self.get('srcNode').appendChild(
                                '<span class="error">Failed!</span>');
                        }
                    }
                });
            });
        }

    });

    var json_error_msg = (
        "The ticket system returned data that could not be understood. " +
        "Please try refreshing, or contact support if this problem " +
        "persists."),

        refresh_msg = "Refreshing data...",

        default_refresh = 60,

        static_prefix = "/static/common/";

        var ticket_status_templates = {
            completed: ('<img src="{static_prefix}check.svg" ' +
                        'title="{title}" />'),
            failed: ('<img src="{static_prefix}cross.svg" ' +
                     'title="{title}"/>'),
            inprogress: ('<img src="{static_prefix}spinner3.gif" ' +
                         'title="{title}"/>'),
            paused: ('<img src="{static_prefix}paused.svg" ' +
                     'title="{title}"/>'),
            'continue': ('<img src="{static_prefix}spinner3.gif" ' +
                         'title="{title}"/>')
        };

    Y.webui = {

        url_prefix: "/api/v1/",

        refresh_interval: default_refresh * 1000,

        refresh_timer: null,

        tempurls: {},

        /**
         * Setup page auto-refresh.
         *
         * Immediately runs the callback when auto-refresh is set up
         * (page is loaded) then use Y.later() for periodically calling
         * the 'callback' at the `Y.webui.refresh_interval` rate.
         *
         * @method setup_auto_refresh
         */
        setup_auto_refresh: function (callback) {
            callback();
            Y.webui.refresh_timer = Y.later(
                Y.webui.refresh_interval, Y, callback, '', true);
        },

        get_workflow_data: function (api_url) {
            var wf_data = {},
                wf_str = {},
                wf_num = {},
                i,
                item;

            Y.io(Y.webui.url_prefix + api_url, {
                sync: true,
                on: {
                    success: function (tx, r) {
                        try {
                            wf_data = Y.JSON.parse(r.responseText).objects;
                        } catch (e) {
                            Y.log(json_error_msg);
                            return;
                        }
                    }
                }
            });

            for (i = 0; i < wf_data.length; i += 1) {
                item = wf_data[i];

                wf_str[item.name] = item.value;
                wf_num[item.value] = item.name;
            }

            return {
                wf_data_list: wf_data,
                wf_data_list_str: wf_str,
                wf_data_list_num: wf_num
            };
        },
        get_workflow_statuses: function () {
            var result, api_url = "ticket_workflow_step_status/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_status_list = result.wf_data_list;
            Y.webui.workflow_status_list_str = result.wf_data_list_str;
            Y.webui.workflow_status_list_num = result.wf_data_list_num;
        },
        get_workflow_steps: function () {
            var result, api_url = "ticket_workflow_step/";

            result = Y.webui.get_workflow_data(api_url);

            Y.webui.workflow_step_list = result.wf_data_list;
            Y.webui.workflow_step_list_str = result.wf_data_list_str;
            Y.webui.workflow_step_list_num = result.wf_data_list_num;
        },
        get_ticket: function (ticket_id, elem) {

            // Load ticket information from TS.
            Y.io(Y.webui.url_prefix + "fullticket/" + ticket_id + "/", {
                sync: true,
                on: {
                    success: function (tx, r) {
                        var data, new_div;

                        try {
                            data = Y.JSON.parse(r.responseText);
                        } catch (e) {
                            Y.log(json_error_msg);
                            return;
                        }

                        new_div = Y.webui.ticket_detail_element(data);
                        if (elem) {
                            elem.setHTML(new_div);
                        }
                        new_div = null;

                    },
                    failure: function (tx, r, e) {
                        Y.log("failure loading fullticket (" +
                              ticket_id + "): ", tx, r, e);
                    }
                }
            });
        },
        basename: function (path) {
            var result = path;

            if (path) {
                result = path.slice(path.lastIndexOf('/') + 1, path.length);
            }

            return result;
        },
        add_artifacts: function (data) {
            var span = Y.Node.create('<span></span>'),
                i;

            for (i = 0; i < data.length; i += 1) {
                span.appendChild(Y.webui.artifact_link(data[i]));
                span.appendChild(Y.Node.create("<br/>"));
            }

            return span;
        },
        artifact_link: function (data, new_name) {
            var name = Y.webui.basename(data.name),
                link = Y.Node.create('<a/>');

            // Lookup a corresponding tempurl.
            var tempurl = Y.webui.tempurls[name];

            if (new_name) {
                name = new_name;
            }
            link.set('text', name);

            // Link the artifact to the tempurl or fallback to the original
            // TS reference.
            if (tempurl === undefined) {
                Y.log('No tempurl for: ' + data.name);
                link.set('href', data.reference);
            } else {
                link.set('href', tempurl);
            }

            return link;
        },

        /**
         * Build a link to a Merge Proposal model ({lp_url: 'lp:<MP ref>'}).
         *
         * Returns a new <a>.
         *
         * @method mp_link
         */

        mp_link: function (mp) {
            var link = Y.Node.create('<a/>')
                .set('text', mp.lp_url)
                .addClass('mp-link');

            var lp_href = mp.lp_url.replace(
                'lp:', 'https://code.launchpad.net/');
            link.set('href', lp_href);

            return link;
        },

        /**
         * Build a link to ppa reference (ppa:<user>/[<distro>/]<name>)
         *
         * Returns a new <a> element for valid reference, otherwise a <span>.
         *
         * @method ppa_link
         */
        ppa_link: function(reference) {
            var matches = reference.match(
                /ppa:([^\/]+)\/([^\/]+)(\/([^\/]+)|)/);
            if (!matches) {
                return Y.Node.create('<span>')
                    .set('text', reference);
            }
            var username = matches[1];
            var distribution = matches[2];
            var name = matches[3];
            if (name) {
                name = name.replace('/', '');
            } else {
                name = distribution;
                distribution = 'ubuntu';
            }
            var href = (
                'https://launchpad.net/~' + username +
                '/+archive/' + distribution + '/' + name);

            return Y.Node.create('<a>')
                .set('href', href)
                .set('text', reference)
                .addClass('ppa-link');
        },
        ticket_detail_element: function (data) {
            var MAX_DESCRIPTION = 200,
                description = "",
                description_p,
                div,
                header_description,
                header_wrapper,
                subticket_wrapper,
                ticket_div,
                ticket_wrapper,
                workflow_inner_div,
                workflow_wrapper;

            if (data.title) {
                div = Y.Node.create("<div id='table_wrapper'/>");

                // These determine the ordering of the divs on the page.
                header_wrapper = div.appendChild(Y.Node.create('<div class="header_wrapper"></div>'));
                workflow_wrapper = div.appendChild(Y.Node.create('<div class="workflow_wrapper"></div>'));
                ticket_wrapper = div.appendChild(Y.Node.create('<div class="ticket_wrapper"></div>'));
                subticket_wrapper = div.appendChild(Y.Node.create('<div class="subticket_wrapper"></div>'));

                var title = Y.Node.create('<h2>')
                    .set('text', data.title);
                header_wrapper.appendChild(title);
                var uuid = Y.Node.create('<h3>');
                uuid.appendChild(
                    Y.Node.create('<small>').set('text', data.uuid));
                header_wrapper.appendChild(uuid);

                if (data.description) {
                    if (data.description.length > MAX_DESCRIPTION) {
                        description = data.description.slice(0, MAX_DESCRIPTION) + "...";
                    } else {
                        description = data.description;
                    }
                }

                header_description = header_wrapper.appendChild(
                    Y.Node.create('<div class="header_description"></div>')
                );
                description_p = header_description.appendChild(Y.Node.create('<p></p>'));
                description_p.appendChild(Y.Node.create('<pre>' + description + '</pre>'));

            }

            // Create a section for presenting ticket status.
            workflow_wrapper.appendChild("<h3>Landing Progress</h3>");
            workflow_inner_div = workflow_wrapper.appendChild(
                Y.Node.create("<div class='workflow'></div>"));

            // Build and render the ticket status datatable.
            var dt_status = Y.webui.get_ticket_status_datatable(data);
            dt_status.render(workflow_inner_div);

            // Create a section for presenting ticket details.
            var status_title = Y.Node.create('<h3>Ticket Details</h3>');
            status_title.appendChild(
                '<a class="edit-link" title="Edit details" href="edit"></a>');
            ticket_wrapper.appendChild(status_title);
            ticket_div = ticket_wrapper.appendChild(
                Y.Node.create("<div class='ticket'></div>"));

            // Build and render the ticket details datatable.
            var dt_details = Y.webui.get_ticket_details_datatable(data);
            dt_details.render(ticket_div);

            // Add an extra row with a link for accessing Lander 'live' logs
            // if the ticket is still in progress.
            if (data.current_workflow_step !==  'Completed' &&
                data.status !== 'Failed') {
                var watch_log = Y.Lang.sub(
                    '<a href="livelog?lander_unit={lander_unit}">' +
                    'Watch Build</a>', data);
                dt_details.addRow({'value': watch_log});
            }

            // Build and render the subtickets datatable.
            var dt_subtickets = Y.webui.get_subtickets_datatable(data);
            dt_subtickets.render(subticket_wrapper);

            return div;
        },

        /**
         * Build a datatable for presenting the given Ticket subtickets.
         *
         * Returns a Y.Datatable loaded with subtickets.
         *
         * @method get_subtickets_datatable
         */
        get_subtickets_datatable: function(ticket) {

            var format_status = function (o) {
                var template_vars = {
                    static_prefix: static_prefix, title: o.data.status};
                var key = o.data.status.replace(' ', '').toLowerCase();
                if (ticket_status_templates.hasOwnProperty(key)) {
                    return Y.Lang.sub(
                        ticket_status_templates[key], template_vars);
                }
                // NEW & 'NOT STARTED'.
                return '-';
            };

            var format_source = function (o) {
                return o.data.sourcepackage.name;
            };

            var format_step = function (o) {
                return o.data.current_workflow_step;
            };

            var format_version = function (o) {
                var last_version = 'Not available'
                var spus = o.data.source_package_uploads;
                if (spus.length > 0) {
                    var last_spu = spus[spus.length - 1];
                    last_version = last_spu.version;
                }
                return last_version
            };

            var format_mps = function (o) {
                var container = Y.Node.create('<div>')
                for (var c in o.data.merge_proposals) {
                    var mp = o.data.merge_proposals[c]
                    container.appendChild(Y.webui.mp_link(mp));
                    container.appendChild('<br>');
                }
                return container.getHTML();
            };

            var format_artifacts = function (o) {
                var container = Y.Node.create('<div>')
                for (var c in o.data.artifact) {
                    var artifact = o.data.artifact[c]
                    container.appendChild(Y.webui.artifact_link(artifact));
                    container.appendChild('<br>');
                }
                return container.getHTML();
            };

            var format_conflicts = function (o) {
                var container = Y.Node.create('<div>')
                for (var c in o.data.conflicts) {
                    var href = o.data.conflicts[c].replace('/api/v1', '');
                    var text = href.replace(
                        '/ticket/', '').replace('/', '');
                    var link = Y.Node.create('<a>')
                        .set('href', href)
                        .set('text', text)
                        .addClass('conflict-link');
                    container.appendChild(link);
                    container.appendChild('<br>');
                }
                return container.getHTML();
            };

            var format_subticket = function (o) {
                // 'sub_columns' preserves the format of a Datatable
                // columns spec just as a fallback plan.
                var sub_columns = [
                    {key: 'source', label: 'Source',
                     allowHTML: true, formatter: format_source,
                     className: 'dt-subticket-source'},
                    {key: 'status', label: 'Status',
                     allowHTML: true, formatter: format_status,
                     className: 'dt-subticket-status'},
                    {key: 'step', label: 'Step',
                     allowHTML: true, formatter: format_step,
                     className: 'dt-subticket-step'},
                    {key: 'version', label: 'Current version',
                     allowHTML: true, formatter: format_version,
                     className: 'dt-subticket-version'},
                    {key: 'merge_proposals', label: 'MPs',
                     allowHTML: true, formatter: format_mps,
                     className: 'dt-subticket-merge-proposals'},
                    {key: 'artifacts', label: 'Artifacts',
                     allowHTML: true, formatter: format_artifacts,
                     className: 'dt-subticket-artifacts'},
                    {key: 'conflicts', label: 'Conflicts',
                     allowHTML: true, formatter: format_conflicts,
                     className: 'dt-subticket-conflicts'},
                ];
                var container = Y.Node.create('<div>');
                var subticket_div = Y.Node.create('<div>')
                    .addClass('subticket');

                var table = subticket_div.appendChild(
                    Y.Node.create('<table>').addClass('subticket'));

                Y.Array.each(sub_columns, function (c){
                    var tr = Y.Node.create('<tr>');
                    tr.appendChild(
                        Y.Node.create('<th>').set('text', c.label + ':'))
                    tr.appendChild(
                        Y.Node.create('<td>').setHTML(c.formatter(o)))
                    table.appendChild(tr);
                });

                container.appendChild(subticket_div);
                return container.getHTML();
            };

            var dt = new Y.DataTable({
                columns: [
                    {key: 'subticket', label: 'Subticket',
                     allowHTML: true, formatter: format_subticket,
                    className: 'dt-outter-subticket'},
                ],
                caption: '<h3>Uploads</h3>',
                data: ticket.subticket
            });


            return dt;
        },

        /**
         * Build a datatable for presenting the given Ticket status.
         *
         * Returns a Y.Datatable loaded with ticket status.
         *
         * @method get_ticket_status_datatable
         */
        get_ticket_status_datatable: function(ticket) {
            var status_map = {
                'PPA creating': {
                    step: 'ppacreator', log_prefix: 'ppacreator'},
                'Package building': {
                    step: 'bsbuilder', log_prefix: 'package_build'},
                'Package validation': {
                    step: 'validator', log_prefix: 'validator'},
                'Image building': {
                    step: 'image_builder', log_prefix: 'image_build'},
                'Image testing': {
                    step: 'test_runner', log_prefix: 'image_test'},
                'Package publishing': {
                    step: 'publisher', log_prefix: 'package_publish'},
                // Saner collection of status, 'step' and 'log_prefix'
                // are status.replace(' ', '_').toLower().
                'Silo creating': {
                    step: 'silo_creating', log_prefix: 'silo_creating'},
                'Silo building': {
                    step: 'silo_building', log_prefix: 'silo_building'},
                'Silo testing': {
                    step: 'silo_testing', log_prefix: 'silo_testing'},
                'Silo publishing': {
                    step: 'silo_publishing', log_prefix: 'silo_publishing'}
            };
            var dataset = [];
            var step_values = {};
            var hidden_steps = Y.Array([
                'NEW', 'COMPLETED', 'FAILED'
            ]);

            // Build a list of steps that are used in this ticket workflow.
            var ticket_workflow_steps = [];
            Y.Array.each(ticket.workflow.steps, function(step) {
                ticket_workflow_steps.push(step.name);
            });

            Y.Array.each(Y.webui.workflow_step_list, function (obj) {
                // Build a map of ticket step values for later use.
                step_values[obj.name] = obj.value;

                // Ignore hidden steps, they don't get a row.
                if (hidden_steps.indexOf(obj.key) !== -1) { return; }

                // Ignore unrelated steps.
                if (status_map.hasOwnProperty(obj.name)) {
                    var step = status_map[obj.name].step;
                    if (ticket_workflow_steps.indexOf(step) === -1) {
                        return;
                    }
                } else if (ticket.citrain_overlay) {
                    // This is the special 'Queued' step, which should not
                    // be rendered for citrain tickets.
                    return;
                }

                // Collect step reviews.
                obj.reviews = [];
                Y.Array.each(ticket.reviews, function(review) {
                    if (review.workflow_step === obj.name) {
                        obj.reviews.push(review);
                    }
                });

                // Collect related ticket/subticket artifacts.
                var prefix = 'misc';
                if (status_map.hasOwnProperty(obj.name)) {
                    prefix = status_map[obj.name].log_prefix;
                }
                function append_local(a) {
                    if (a.name.slice(0, prefix.length) == prefix) {
                        obj.logs.push(a);
                    }
                }
                obj.logs = [];
                Y.Array.each(ticket.artifact, append_local);
                Y.Array.each(ticket.subticket, function(s) {
                    Y.Array.each(s.artifact, append_local);
                });

                // Create a new dt row.
                dataset.push(obj);
            });

            var current_step_value = step_values[
                ticket.current_workflow_step];

            var format_step = function (o) {
                return o.data.name + ':';
            };

            var format_status = function (o) {
                var template_vars = {
                    static_prefix: static_prefix, title: o.data.name};
                var value;
                if (o.data.value < current_step_value) {
                    // Past steps always 'Passed'.
                    value = Y.Lang.sub(
                        ticket_status_templates.completed, template_vars);
                } else if (o.data.value === current_step_value) {
                    // Current step according to the ticket 'status'.
                    o.rowClass = 'dt-status-current';
                    var key = ticket.status.replace(' ', '').toLowerCase();
                    if (ticket_status_templates.hasOwnProperty(key)) {
                        value = Y.Lang.sub(
                            ticket_status_templates[key], template_vars);
                    } else {
                        // Just render the text of unexpected statuses.
                        value = ticket.status;
                    }
                } else {
                    // Steps yet to come are empty.
                    value = '';
                }

                value += '</br>'

                // Render a placeholder for related reviews recording
                // the dataset 'row-id' as an extra attribute.
                var c = Y.Node.create('<div>');
                c.appendChild(
                    Y.Node.create('<div class="reviews">')
                        .setAttribute('row-id', o.rowIndex));
                value += c.getHTML();

                // Render related artifacts links with shortened labels.
                Y.Array.each(o.data.logs, function (log) {
                    var label = log.name.slice(log.name.indexOf('.') + 1);
                    var link = Y.webui.artifact_link(log, label);
                    var c = Y.Node.create('<div>');
                    c.appendChild(link);
                    value += c.getHTML() + '</br>';
                });

                return value;
            };

            var dt = new Y.DataTable({
                columns: [
                    {key: 'step', label: 'Step', 'width': '250%',
                     allowHTML: false, formatter: format_step,
                     className: 'dt-status-name'},
                    {key: 'status', label: 'Status',
                     allowHTML: true, formatter: format_status,
                     className: 'dt-status-value'}
                ],
                data: dataset
            });

            dt.after('render', function (e) {
                // XXX cprov 2014-10-31: halloween treat, 'dt:render' seems
                // to be fired before the rows are rendered ... Y.later
                // just introduces the *needed* delay for rendering the
                // inline reviews.
                Y.later(0, Y, function () {
                    Y.all('.reviews').each(function (node) {
                        var row_id = Y.Number.parse(
                            node.getAttribute('row-id'));
                        var record = e.currentTarget.getRecord(row_id);
                        Y.Array.each(record.get('reviews'), function (r) {
                            var w = new InlineReview({
                                boundingBox: node,
                                review: r,
                                readonly: (ticket.current_workflow_step !==
                                           r.workflow_step)
                            });
                            w.render();
                        });
                    });
                }, '', false);
            })

            return dt;
        },
        /**
         * Build a datatable for presenting details of the given Ticket.
         *
         * Returns a Y.Datatable loaded with ticket details.
         *
         * @method get_ticket_details_datatable
         */
        get_ticket_details_datatable: function(ticket) {

            var dataset = [];
            var fields = [
                'artifact',
                'creator', 'landers', 'created', 'updated', 'private',
                'bug_id', 'sync_request',
                'series', 'base_image', 'master_ppa', 'build_ppa',
                'job_url', 'test_notes', 'comments',
            ];

            if (ticket.citrain_overlay) {
                ticket = Y.merge(ticket, ticket.citrain_overlay);
            }

            Y.Array.each(fields, function (key) {
                if (!ticket.hasOwnProperty(key)) return;
                dataset.push({'key': key, 'value': ticket[key]});
            });

            var format_key = function (o) {
                if (!o.data.key) return '';
                var text = o.data.key.replace('_', ' ');
                return text.charAt(0).toUpperCase() + text.slice(1) + ':';
            }

            var format_value = function (o) {
                var value;
                switch (o.data.key) {
                    case 'artifact':
                        value = Y.webui.add_artifacts(o.data.value).getHTML();
                        break;
                    case 'bug_id':
                        if (!o.data.value) {
                           break;
                        }
                        value = Y.Lang.sub(
                            '<a class="bug-link" ' +
                            'href="https://launchpad.net/bugs/{bug_id}">' +
                            '#{bug_id}</a>',
                            {bug_id: o.data.value});
                        break;
                    case 'base_image':
                        var label = o.data.value.slice(
                            o.data.value.lastIndexOf('/') + 1);
                        value = Y.Lang.sub(
                            '<a class="image-link" href="{href}">{label}</a>',
                            {href: o.data.value, label: label});
                        break;
                    case 'comments':
                    case 'test_notes':
                        value = Y.Lang.sub(
                            '<textarea readonly style="width: 80%;">' +
                            '{content}</textarea>',
                            {content: o.data.value});
                        break;
                    case 'job_url':
                        if (!o.data.value) {
                           break;
                        }
                        var label = o.data.value;
                        value = Y.Lang.sub(
                            '<a class="job-link" href="{href}">{label}</a>',
                            {href: o.data.value, label: label});
                        break;
                    case 'build_ppa':
                    case 'master_ppa':
                        var n = Y.Node.create('<div>');
                        n.appendChild(Y.webui.ppa_link(o.data.value));
                        value = n.getHTML();
                        break;
                    default:
                        value = o.data.value;
                }
                return value;
            }

            return new Y.DataTable({
                columns: [
                    {key: 'key', label: 'Key', 'width': '100%',
                     allowHTML: true, formatter: format_key,
                     className: 'dt-details-key'},
                    {key: 'value', label: 'Value',
                     allowHTML: true, formatter: format_value,
                     className: 'dt-details-value'}
                ],
                data: dataset
            });
        },

        /**
         * Load a map of tempurls for the context artifacts from GK.
         *
         * Reset Y.webui.tempurls.
         *
         * @method load_tempurls
         */
        load_tempurls: function(ticket_id_or_uuid) {
            Y.webui.tempurls = {};
            Y.io("/gk/api/v1/ticket/" + ticket_id_or_uuid, {
                sync: true,
                on: {
                    success: function (tx, r) {
                        Y.webui.tempurls = Y.JSON.parse(r.responseText);
                    },
                    failure: function (tx, r, e) {
                        Y.log("Failure loading tempurls (" +
                              ticket_id_or_uuid + "): ", tx, r, e);
                    }
                }
            });
        },

        /**
         * Returns the ticket ID based on the window URL.
         *
         * Checks the known ticket page path (/ticket/<id>), then match
         * a valid integer ID.
         *
         * @method get_ticket_id
         */
        get_ticket_id: function () {
            var path = Y.config.win.location.pathname;

            // Ensure the URL is what we expect for the ticket page.
            var uuid_pattern = '[0-9a-f]{8}-([0-9a-f]{4}-){3}[0-9a-f]{12}'
            var url_regexp = new RegExp(
                '\/ticket\/(\\d+|' + uuid_pattern + ')\/', 'g')
            if (!path.match(url_regexp)) {
                return null;
            }

            // Isolate given term (ID or UUID);
            var ticket_id_or_uuid = path.replace('/ticket/', '');
            ticket_id_or_uuid = ticket_id_or_uuid.replace('/', '');

            // Try to load tempurls. It will fail if the given term does
            // not match the one used for creating the swift container.
            Y.webui.load_tempurls(ticket_id_or_uuid);
            if (Object.keys(Y.webui.tempurls).length > 0 ) {
                return ticket_id_or_uuid;
            }

            // Let's approach TS and use another alternative.
            Y.io("/api/v1/ticket/" + ticket_id_or_uuid + '/', {
                sync: true,
                on: {
                    success: function (tx, r) {
                        var t = Y.JSON.parse(r.responseText);
                        if (ticket_id_or_uuid.match(
                                new RegExp(uuid_pattern, 'g'))) {
                            ticket_id_or_uuid = t['id'];
                        } else {
                            ticket_id_or_uuid = t['uuid'];
                        }
                    },
                    failure: function (tx, r, e) {
                        Y.log("Failure finding ticket alternative key (" +
                              ticket_id_or_uuid + "): ", tx, r, e);
                    }
                }
            });

            // Now we can properly load tempurls and continue safely.
            Y.webui.load_tempurls(ticket_id_or_uuid);
            return ticket_id_or_uuid;
        },

        /**
         * Updates ticket information.
         *
         * @method update_ticket
         */
        update_ticket: function () {
            var ticket_id = Y.webui.get_ticket_id();
            if (ticket_id) {
                Y.webui.get_ticket(ticket_id, Y.one('#container'));
            }
        },

        /**
         * Setup a DataTable and a remote DataSource for listing tickets.
         *
         * 'type' can be one of 'open', 'complete', 'failed', otherwise
         * all tickets will be listed.
         *
         * @method setup_ticket_datatable
         */
        setup_ticket_datatable: function (container, type) {
            var itemsPerPage = 10;

            // Activate the list context link.
            Y.all('.breadcrumb li.' + type).addClass('active');

            var source = Y.webui.url_prefix + 'ticket/';

            // Adjust datasource query arguments according to the
            // ticket list 'type'.
            var query = '?order_by=-id&page={page}&limit={itemsPerPage}&';
            var initial = '?order_by=-id&limit=' + itemsPerPage + '&';
            var clause = '';
            if (type === 'open') {
                // Tickets QUEUED, PAUSED, RESUMED, INPROGRESS.
                clause = 'status__lte=100';
                query += clause;
                initial += clause
            } else if (type === 'complete') {
                // Tickets COMPLETED.
                clause = 'status=200';
                query += clause;
                initial += clause
            } else if (type === "failed") {
                // Tickets FAILED.
                clause = 'status=300';
                query += clause;
                initial += clause
            } else if (type === "pending-qa") {
                // Tickets Pending-QA.
                clause = 'reviews__review_type=QA&reviews__status=0';
                query += clause;
                initial += clause
            }

            // Paginator container.
            var paginator = Y.Node.create('<div>')
                .set('id', 'paginator');
            Y.one('#main-content').appendChild(paginator);

            // Add new ticket content.
            var new_ticket_link = Y.Node.create(
                '<a id="add-ticket" href="create" ' +
                    'class="button right space-top"' +
                    '>+ Add Ticket</a>'
            );
            Y.one('#main-content').appendChild(new_ticket_link);

            // Row formatters ...
            var link_it = function (o) {
                return '<a class="row_link" href="/ticket/' +
                    o.data.uuid + '/">' + o.value + '</a>';
            };
            var format_id = function (o) {
                o.rowClass = "data-row-" + o.data.uuid;
                return link_it(o);
            };

            var format_step = function (o) {
                var c = Y.Node.create('<div>');

                var template_vars = {
                    static_prefix: static_prefix, title: o.data.status};
                var key = o.data.status.replace(' ', '').toLowerCase();
                if (ticket_status_templates.hasOwnProperty(key)) {
                    var icon = Y.Lang.sub(
                        ticket_status_templates[key], template_vars);
                    c.appendChild(icon);
                } else {
                    c.appendChild('<span style="padding: 12px;"/>');
                }
                c.appendChild(link_it(o));

                return c.getHTML();
            };

            var format_reviews = function (o) {
                var container = Y.Node.create('<div>')
                    .addClass('yui3-inlinereview-content');
                o.data.reviews.forEach( function(r) {
                    var label = Y.Node.create('<label>')
                        .set('text', r.review_type)
                        .addClass(r.status.toLowerCase());
                    container.appendChild(label);
                    container.appendChild('<span style="padding: 5px;"/>');
                });
                var c = Y.Node.create('<div>');
                c.appendChild(container);
                return c.getHTML();
            };

            var dt = new Y.DataTable({
                columnset: [
                    {key: 'title', label: 'Title',
                     formatter: format_id, allowHTML: true},
                    {key: 'current_workflow_step', label: 'Landing Progress',
                     formatter: format_step, allowHTML: true},
                    {key: 'reviews', label: 'Review Status',
                     formatter: format_reviews, allowHTML: true}
                ],
                paginator: new Y.PaginatorView({
                    model: new Y.PaginatorModel({
                        itemsPerPage: itemsPerPage,
                    }),
                    maxPageLinks: 2,
                    container: '#paginator',
                }),
                resizePaginator: true,
                paginationSource: 'server',
                requestStringTemplate: query,
                strings: {
                    'emptyMessage': 'No results available.'
                }
            });

            var ds = new Y.DataSource.IO({
                source: source,
            });

            ds.on('response', function(e) {
                if (e.data.status !== 200) {
                    Y.log('Datasource loading failure!' + e);
                }
            });

            ds.plug(Y.Plugin.DataSourceJSONSchema, {
                schema: {
                    resultListLocator: 'objects',
                    resultFields: [
                        'title', 'current_workflow_step', 'status', 'uuid',
                        'reviews'
                    ],
                    metaFields: {
                        totalItems: 'meta.total_count'
                    }
                }
            });

            dt.plug(Y.Plugin.DataTableDataSource, {
                datasource: ds,
                initialRequest: initial
            });

            // Setup a periodic refreshing tasks.
            Y.later(Y.webui.refresh_interval, Y, function () {
               dt.refreshPaginator();
            }, '', true);

            // Finally render the ticket list.
            dt.render(container);
        }
    };

    Y.webui.get_workflow_steps();
    Y.webui.get_workflow_statuses();
}, '0.0.1', {requires: ['datatable', 'datatype-number', 'io', 'json-parse',
                       'json-stringify', 'overlay', 'widget-autohide']});
