"use strict";

// Make jslint happy
var YUI;

YUI.add('webuiforms', function (Y) {

    /**
     * Overriding Y.SelectField for properly rendering '<label>'s.
     */
    Y.WebUISelectField = Y.Base.create(
        'select-field', Y.SelectField, [Y.WidgetParent, Y.WidgetChild], {

        _renderLabelNode : function () {
            var contentBox = this.get('contentBox'),
                titleNode = Y.Node.create('<label></label>');
            titleNode.set('innerHTML', this.get('label'));
            contentBox.appendChild(titleNode);
            this._labelNode = titleNode;
        }
    });

    Y.webuiforms = {

        /**
         * Hardcoded array of series choices, must be kept in sync with
         * the supported options in Ticket-System.
         */
        seriesChoices: [
            {label : 'Vivid - 15.04', value : 'vivid'},
            {label : 'Utopic - 14.10', value : 'utopic'},
            {label : 'Trusty - 14.04', value : 'trusty'},
            {label : 'Precise - 12.04', value : 'precise'}
        ],

        /**
         * Convert form 'sources' to 'subtickets'.
         *
         * @method _mangleFormSources
         */
        _mangleFormSources: function (form_data) {
            var sources_list = form_data['sources'].split('\n');
            delete form_data['sources'];
            form_data['subtickets'] = [];
            sources_list.forEach( function(name) {
                if (!Y.Lang.trim(name)) { return; }
                form_data['subtickets'].push({
                    'sourcepackage': {'name': name},
                    'assignee': form_data['owner']
                });
            });
            return form_data;
        },

        /**
         * Convert form 'citrain' attributes to 'citrain_overlay'.
         *
         * @method _mangleFormCitrain
         */
        _mangleFormCitrain: function (form_data) {
            var citrain_overlay_fields = [
                'comments', 'test_notes', 'landers', 'sync_request',
                'job_url'
            ]
            var citrain_overlay_obj = {}
            citrain_overlay_fields.forEach( function(name) {
                citrain_overlay_obj[name] = form_data[name];
                delete form_data[name];
            });
            form_data['citrain_overlay'] = citrain_overlay_obj;
            return form_data;
        },

        /**
         * Add appropiate reviews to the form.
         *
         * @method _mangleFormReviews
         */
        _mangleFormReviews: function (form_data) {
            form_data['reviews'] = [
                {'review_type': 'Developer',
                 'workflow_step': 'Silo testing'}
            ];
            if (form_data['qa_review']) {
                form_data['reviews'].push({
                    'review_type': 'QA',
                    'workflow_step': 'Silo testing'
                });
            };
            delete form_data['qa_review'];
            return form_data;
        },

        /**
         * Setup a form for creating a new ticket.
         *
         * Uses Y.Form() to build and render the form dynamically and
         * re-implements its submission to POST JSON data (instead of
         * 'urlencoded'). Opens the just-created ticket on success.
         *
         * @method setupCreateForm
         */
        setupCreateForm: function (container) {
            var form_message = Y.Node.create('<div>');
            Y.one(container).appendChild(form_message);

            // Build and render a form for creating a new Ticket.
            var f = new Y.Form({
                boundingBox: container,
                action : '/api/v1/ticket/',
                method : 'post',
                children : [
                    {name: 'title', required: true, label: 'Title:'},
                    {name: 'description', label: 'Description:',
                     type: 'TextareaField'},
                    {name: 'owner', required: true, label: 'Owner:',
                     validator: Y.FormField.VALIDATE_EMAIL_ADDRESS},
                    {name: 'landers', label: 'Landers:'},
                    {name: 'qa_review', label: 'QA sign-off required?',
                     type: 'CheckboxField'},
                    {name: 'sync_request', label: 'Sync request:'},
                    {name: 'series', required: true, label: 'Series:',
                     type: 'WebUISelectField', useDefaultOption: false,
                     choices: this.seriesChoices},
                    {name: 'sources', label: 'Source Packages:',
                     type: 'TextareaField'},
                    {name: 'test_notes', label: 'Test notes:',
                     type: 'TextareaField'},
                    {name: 'comments', label: 'Comments:',
                     type: 'TextareaField'},
                    {name: 'create-button', type: 'SubmitButton',
                     value: 'Create!'}
                ]
            });

            f.render();
            Y.one('.yui3-submit-button-content').appendChild(
                Y.Node.create('<a>')
                    .set('text', 'Cancel')
                    .set('href', '.')
            );

            // Hijack the default (www-urlencoded) form submission and
            // submit its content as JSON.
            Y.one('input[name="create-button"]').on('click', function(e) {
                e.halt();
                if (!f._runValidation()) {
                    return 1;
                }

                var form_data = f.toJSON();
                form_data = Y.webuiforms._mangleFormSources(form_data);
                form_data = Y.webuiforms._mangleFormCitrain(form_data);
                form_data = Y.webuiforms._mangleFormReviews(form_data);
                form_data['workflow'] = {'name': 'citrain'};

                Y.io(f.get('action'), {
                    method: f.get('method'),
                    sync: true,
                    data: Y.JSON.stringify(form_data),
                    headers: {
                        'Content-Type': 'application/json;',
                    },
                    on: {
                        success: function(tx, r) {
                            var location = r.getResponseHeader('Location');
                            Y.config.win.location.href = location.replace(
                                '/api/v1', '');
                        },
                        failure: function (tx, r) {
                            Y.log('FAILURE: ' + r.responseText);
                            form_message.addClass('form-error')
                            form_message.set(
                                'text', 'Ticket creation failed!')
                        }
                    }
                });
            });
        },
        /**
         * Setup a form for editing a ticket.
         *
         * @method setupEditForm
         */
        setupEditForm: function (container) {
            var form_message = Y.Node.create('<div>');
            Y.one(container).appendChild(form_message);

            // Build and render a form for editing Ticket.
            var f = new Y.Form({
                boundingBox: container,
                action : '/api/v1/ticket/' + ticket_id + '/',
                method : 'post',
                children : [
                    {name: 'title', required: true, label: 'Title:'},
                    {name: 'owner', required: true, label: 'Owner:',
                     validator: Y.FormField.VALIDATE_EMAIL_ADDRESS},
                    {name: 'landers', label: 'Landers:'},
                    {name: 'bug_id', label: 'Bug id:'},
                    {name: 'sync_request', label: 'Sync request:'},
                    {name: 'sources', label: 'Source Packages:',
                     type: 'TextareaField'},
                    {name: 'description', label: 'Description:',
                     type: 'TextareaField'},
                    {name: 'test_notes', label: 'Test notes:',
                     type: 'TextareaField'},
                    {name: 'comments', label: 'Comments:',
                     type: 'TextareaField'},
                    {name: 'edit-button', type: 'SubmitButton',
                     value: 'Update!'}
                ]
            });

            // Load ticket information from TS and pre-fill the form.
            Y.io('/api/v1/fullticket/' + ticket_id + '/', {
                sync: true,
                on: {
                    success: function (tx, r) {
                        var ticket = Y.JSON.parse(r.responseText);
                        if (ticket.citrain_overlay) {
                            ticket = Y.merge(ticket, ticket.citrain_overlay);
                        }
                        f.each( function(field) {
                            var name = field.get('name');
                            switch (name) {
                            case 'edit-button':
                                return;
                            case 'bug_id':
                                field.set(
                                    'value', (ticket[name] || '').toString());
                                break;
                            case 'sources':
                                var sources = [];
                                ticket.subticket.forEach( function(s) {
                                    sources.push(s.sourcepackage.name);
                                });
                                field.set('value', sources.join('\n'));
                                break;
                            default:
                                field.set('value', ticket[name] || '');
                            }
                        });
                    },
                    failure: function (tx, r, e) {
                        console.log("failure loading fullticket (" +
                                    ticket_id + "): ", tx, r, e);
                    }
                }
            });

            // Render the form and a 'Cancel' link.
            f.render();
            Y.one('.yui3-submit-button-content').appendChild(
                Y.Node.create('<a>')
                    .set('text', 'Cancel')
                    .set('href', '.')
            );

            Y.one('input[name="edit-button"]').on('click', function(e) {
                e.halt();
                if (!f._runValidation()) {
                    return 1;
                }
                var form_data = f.toJSON();
                form_data = Y.webuiforms._mangleFormSources(form_data);
                form_data = Y.webuiforms._mangleFormCitrain(form_data);

                Y.io(f.get('action'), {
                    method: 'patch',
                    sync: true,
                    data: Y.JSON.stringify(form_data),
                    headers: {
                        'Content-Type': 'application/json;',
                    },
                    on: {
                        success: function(tx, r) {
                            Y.config.win.location.href = f.get(
                                'action').replace('/api/v1', '');
                        },
                        failure: function (tx, r) {
                            Y.log('FAILURE: ' + r.responseText);
                            form_message.addClass('form-error')
                            form_message.set(
                                'text', 'Ticket update failed!')
                        }
                    }
                });
            });
        }
    };
}, '0.0.1', {requires: ['io', 'node', 'json-parse', 'json-stringify',
                        'gallery-form']});
