"""
Django settings for webui project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

import json
import os
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'x8j-r2u(_l_$z3-_m4q-8llzz05i7*7(eu5mhd^p5ph!s$1j*r'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = True

ALLOWED_HOSTS = []


# Application definition
LOCAL_APPS = (
    'common',
    'status',
    'tickets',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # Only included to provide static files for Ticket System admin/ proxying.
    'django.contrib.admin',
) + LOCAL_APPS

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'webui.urls'

WSGI_APPLICATION = 'webui.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = 'static'

try:
    p = os.path.join(BASE_DIR, 'local_settings.py')
    if os.path.exists(p):
        execfile(p)
except ImportError:
    pass

allowed = os.path.abspath(
    os.path.join(BASE_DIR, '../../../../allowed_hosts.json'))
if os.path.exists(allowed):
    with open(allowed) as f:
        ALLOWED_HOSTS = json.load(f)
    print('ALLOWED_HOSTS from (%s): %s' % (allowed, ', '.join(ALLOWED_HOSTS)))

logging = os.path.join(BASE_DIR, '../../../logging.json')
if os.path.exists(logging):
    data = {}
    with open(logging) as f:
        data = json.load(f)

    log_dir = data.get('log_dir')
    log_level = data.get('log_level')

    if log_dir and os.path.exists(log_dir) and log_level:
        log_file = os.path.join(log_dir, 'django.log')
        LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'formatters': {
                'standard': {
                    'format': (
                        "[%(asctime)s] %(levelname)s [%(name)s"
                        ":%(lineno)s] %(message)s"),
                    'datefmt': "%d/%b/%Y %H:%M:%S"
                },
            },
            'handlers': {
                'file': {
                    'level': log_level,
                    'class': 'logging.FileHandler',
                    'filename': log_file,
                    'formatter': 'standard',
                },
            },
            'loggers': {
                'django': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
                '': {
                    'handlers': ['file'],
                    'level': log_level,
                    'propagate': True,
                },
            },
        }
